/*******************************************************************************
* Copyright (C) 2013 Spansion LLC. All Rights Reserved. 
*
* This software is owned and published by: 
* Spansion LLC, 915 DeGuigne Dr. Sunnyvale, CA  94088-3453 ("Spansion").
*
* BY DOWNLOADING, INSTALLING OR USING THIS SOFTWARE, YOU AGREE TO BE BOUND 
* BY ALL THE TERMS AND CONDITIONS OF THIS AGREEMENT.
*
* This software contains source code for use with Spansion 
* components. This software is licensed by Spansion to be adapted only 
* for use in systems utilizing Spansion components. Spansion shall not be 
* responsible for misuse or illegal use of this software for devices not 
* supported herein.  Spansion is providing this software "AS IS" and will 
* not be responsible for issues arising from incorrect user implementation 
* of the software.  
*
* SPANSION MAKES NO WARRANTY, EXPRESS OR IMPLIED, ARISING BY LAW OR OTHERWISE,
* REGARDING THE SOFTWARE (INCLUDING ANY ACOOMPANYING WRITTEN MATERIALS), 
* ITS PERFORMANCE OR SUITABILITY FOR YOUR INTENDED USE, INCLUDING, 
* WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY, THE IMPLIED 
* WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE OR USE, AND THE IMPLIED 
* WARRANTY OF NONINFRINGEMENT.  
* SPANSION SHALL HAVE NO LIABILITY (WHETHER IN CONTRACT, WARRANTY, TORT, 
* NEGLIGENCE OR OTHERWISE) FOR ANY DAMAGES WHATSOEVER (INCLUDING, WITHOUT 
* LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS, BUSINESS INTERRUPTION, 
* LOSS OF BUSINESS INFORMATION, OR OTHER PECUNIARY LOSS) ARISING FROM USE OR 
* INABILITY TO USE THE SOFTWARE, INCLUDING, WITHOUT LIMITATION, ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL OR CONSEQUENTIAL DAMAGES OR LOSS OF DATA, 
* SAVINGS OR PROFITS, 
* EVEN IF SPANSION HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. 
* YOU ASSUME ALL RESPONSIBILITIES FOR SELECTION OF THE SOFTWARE TO ACHIEVE YOUR
* INTENDED RESULTS, AND FOR THE INSTALLATION OF, USE OF, AND RESULTS OBTAINED 
* FROM, THE SOFTWARE.  
*
* This software may be replicated in part or whole for the licensed use, 
* with the restriction that this Disclaimer and Copyright notice must be 
* included with each copy of this software, whether used in part or whole, 
* at all times.  
*/
/******************************************************************************/
/** \file main.c
 **
 ** Main Module
 ** 
 ** \brief This example shows how to use the ADC with DMA transfer.
 **
 ** This example set the ADC0 channel 0 in repeat conversion mode and output 
 ** interrupt request to DMA  channel 0.
 ** Via software trigger the conversion is started, when the scan interrupt 
 ** occurs, the ADC result data is transferred from ADC FIFO to a global array 
 ** variable via DMA bus automatically. After the transfer time comes to 
 ** DMA_MAX_COUNT, DMA interrupt request will issue. In the callback of DMA
 ** interrupt request, ADC repeat conversion is stopped.
 **
 ** History:
 **   - 2014-03-04  1.0  Edison Zhang    First version.
 **
 ******************************************************************************/

/******************************************************************************/
/* Include files                                                              */
/******************************************************************************/
#include "pdl_header.h"

#define DMA_MAX_COUNT 256u

/******************************************************************************/
/* Global variable                                                            */
/******************************************************************************/
boolean_t bDmaFinished = FALSE;			    // DMA finished notification flag
uint16_t  au16DestinationData[DMA_MAX_COUNT]; // Destination data (filled in main() )

/**
 ******************************************************************************
 ** \brief  DMA Error callback function
 ******************************************************************************/
void Main_dma_error_callback(uint8_t u8ErrorCode)
{
    // Error handling ... Should never happen.
}

/**
 ******************************************************************************
 ** \brief  DMNA finish callback function
 ******************************************************************************/
void Main_dma_finish_callback(void)
{
    Adc_StopScanRepeat(&ADC0);   // Stop ADC scan repeat conversion
    
    bDmaFinished = TRUE;         // Set DMA finished notification flag
}

/**
 ******************************************************************************
 ** \brief  DMA configuration 
 ******************************************************************************/
void Main_Dma_Config(void)
{
    stc_dma_config_t stcConfigDma;
    stc_dma_int_sel_t stcIntSel;
    stc_dma_int_cb_t  stcIntCb;

    // Clear local configuration structure to zero.
    PDL_ZERO_STRUCT(stcConfigDma);	
    PDL_ZERO_STRUCT(stcIntSel);	
    PDL_ZERO_STRUCT(stcIntCb);	

    // DMAC0 configuration
    stcConfigDma.enDmaIdrq = Adc0;
    stcConfigDma.u8BlockCount = 1u;
    stcConfigDma.u16TransferCount = DMA_MAX_COUNT ;   
    stcConfigDma.enTransferMode = DmaDemandTransfer;
    stcConfigDma.enTransferWdith = Dma16Bit;
    stcConfigDma.u32SourceAddress = ADC0_SCAN_FIFO_ADDR + 2;			        // ADC result data address
    stcConfigDma.u32DestinationAddress = (uint32_t) &au16DestinationData[0u];	// Destination array's address
    stcConfigDma.bFixedSource = TRUE;
    stcConfigDma.bFixedDestination = FALSE;
    stcConfigDma.bReloadCount = FALSE;     
    stcConfigDma.bReloadSource = FALSE;
    stcConfigDma.bReloadDestination = FALSE;
    stcConfigDma.bEnableBitMask = FALSE;

    if (Ok == Dma_Init_Channel(0u, &stcConfigDma))      // Initialize DMA channel 0
    {
        /* Configure interrupt */
        stcIntSel.bCompleteInt = 1;
        stcIntSel.bErrorInt = 1;
        stcIntCb.pfnCompleteCallback = Main_dma_finish_callback;
        stcIntCb.pfnErrorCallback = Main_dma_error_callback; 
        Dma_EnableInt(0u, &stcIntSel, &stcIntCb);

        Dma_Enable();            // Overall enable of DMA

        Dma_SetChannel(0u, TRUE, FALSE, FALSE);	      // Enable channel
    }
}

/**
 ******************************************************************************
 ** \brief  Error callback function
 ******************************************************************************/
void Main_adc_error_callback(void)
{
    while(1); // Should never enter...
}

/**
 ******************************************************************************
 ** \brief  Result callback function
 **
 ** \param  u32AdcArgument  ADC FIFO value
 ******************************************************************************/
void Main_adc_result_callback(uint32_t* pu32AdcArgument)
{
    while(1); // Should never enter...
}

/**
 ******************************************************************************
 ** \brief  ADC0 configuration
 ******************************************************************************/
void Main_Adc_Config(void)
{
    stc_adcn_t*      pstcAdc = (stc_adcn_t*) &ADC0;
    stc_adc_config_t stcConfig;
    stc_adc_int_sel_t stcIntSel;
    stc_adc_int_callback_t stcIntCallback;

    PDL_ZERO_STRUCT(stcConfig);		// Clear local configuration to zero.

    /* Common setting */
    stcConfig.bLsbAlignment = TRUE;
    stcConfig.u32SamplingTimeSelect.AD_CH_0 = 1u;
    stcConfig.enSamplingTimeN0 = Value32;
    stcConfig.u8SamplingTime0 = 30u;
    stcConfig.enSamplingTimeN1 = Value32;
    stcConfig.u8SamplingTime1 = 30u;
    stcConfig.u8SamplingMultiplier = 5u;
    stcConfig.u8EnableTime = 10u;   
    stcConfig.bLsbAlignment = TRUE; // Place data to LSB of high 16 bits of SCFD
    /* Scan conversion configuration */
    stcConfig.bScanInit = TRUE;
    stcConfig.stcScanInit.u32ScanCannelSelect.AD_CH_0 = 1u;
    stcConfig.stcScanInit.enScanMode = ScanRepeatConversion;
    stcConfig.stcScanInit.bScanTimerStartEnable = FALSE;
    stcConfig.stcScanInit.u8ScanFifoDepth = 0u;       // Only one conversion
    stcConfig.stcScanInit.enScanTimerTrigger = AdcNoTimer;
    /* Priority converison is not used, so don't initialize it */
    stcConfig.bPrioInit = FALSE;
    /* Comparison function is not used, so don't initialize it */
    stcConfig.bComparisonEnable = FALSE;
    /* Range comparison function is not used, so don't initialize it */
    stcConfig.bRangeComparisonEnable = FALSE;
    
    if (Ok == Adc_Init(pstcAdc, &stcConfig))        // Init ADC0
    {
        /* Configure interrupt  */
        PDL_ZERO_STRUCT(stcIntSel);
        PDL_ZERO_STRUCT(stcIntCallback);
        stcIntSel.bScanIrq = 1;
        stcIntCallback.pfnScanCallback = (func_ptr_adc_parg32_t)Main_adc_result_callback;
        Adc_EnableInt(pstcAdc, &stcIntSel, &stcIntCallback);
        
        Adc_Enable(pstcAdc);               // Enable ADC0 and wait for ready
    }
}

/**
 ******************************************************************************
 ** \brief  Main function
 **
 ** \return int32_t return value, if needed
 ******************************************************************************/
int32_t main(void)
{
    Main_Dma_Config();
    Main_Adc_Config();

    Adc_SwTriggerScan(&ADC0);        // Trigger ADC0
    
    while(FALSE == bDmaFinished)  // Wait for interrupt notification
    {}

    Adc_DeInit((stc_adcn_t*) &ADC0);
    Dma_DeInit_Channel(0u);

    while(1)
    {}
  
}
