/*******************************************************************************
* Copyright (C) 2013 Spansion LLC. All Rights Reserved. 
*
* This software is owned and published by: 
* Spansion LLC, 915 DeGuigne Dr. Sunnyvale, CA  94088-3453 ("Spansion").
*
* BY DOWNLOADING, INSTALLING OR USING THIS SOFTWARE, YOU AGREE TO BE BOUND 
* BY ALL THE TERMS AND CONDITIONS OF THIS AGREEMENT.
*
* This software contains source code for use with Spansion 
* components. This software is licensed by Spansion to be adapted only 
* for use in systems utilizing Spansion components. Spansion shall not be 
* responsible for misuse or illegal use of this software for devices not 
* supported herein.  Spansion is providing this software "AS IS" and will 
* not be responsible for issues arising from incorrect user implementation 
* of the software.  
*
* SPANSION MAKES NO WARRANTY, EXPRESS OR IMPLIED, ARISING BY LAW OR OTHERWISE,
* REGARDING THE SOFTWARE (INCLUDING ANY ACOOMPANYING WRITTEN MATERIALS), 
* ITS PERFORMANCE OR SUITABILITY FOR YOUR INTENDED USE, INCLUDING, 
* WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY, THE IMPLIED 
* WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE OR USE, AND THE IMPLIED 
* WARRANTY OF NONINFRINGEMENT.  
* SPANSION SHALL HAVE NO LIABILITY (WHETHER IN CONTRACT, WARRANTY, TORT, 
* NEGLIGENCE OR OTHERWISE) FOR ANY DAMAGES WHATSOEVER (INCLUDING, WITHOUT 
* LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS, BUSINESS INTERRUPTION, 
* LOSS OF BUSINESS INFORMATION, OR OTHER PECUNIARY LOSS) ARISING FROM USE OR 
* INABILITY TO USE THE SOFTWARE, INCLUDING, WITHOUT LIMITATION, ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL OR CONSEQUENTIAL DAMAGES OR LOSS OF DATA, 
* SAVINGS OR PROFITS, 
* EVEN IF SPANSION HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. 
* YOU ASSUME ALL RESPONSIBILITIES FOR SELECTION OF THE SOFTWARE TO ACHIEVE YOUR
* INTENDED RESULTS, AND FOR THE INSTALLATION OF, USE OF, AND RESULTS OBTAINED 
* FROM, THE SOFTWARE.  
*
* This software may be replicated in part or whole for the licensed use, 
* with the restriction that this Disclaimer and Copyright notice must be 
* included with each copy of this software, whether used in part or whole, 
* at all times.  
*/
/******************************************************************************/
/** \file main.c
 **
 ** Main Module for Dual Timer sample (using interrupt)
 **
 ** History:
 **   - 2014-02-16  0.0.1  Edison Zhang         First version.
 **
 ******************************************************************************/

/******************************************************************************/
/* Include files                                                              */
/******************************************************************************/
#include "pdl_header.h"

/******************************************************************************/
/* Local pre-processor symbols/macros ('#define')                             */
/******************************************************************************/

/******************************************************************************/
/* Global variable definitions (declared in header file with 'extern')        */
/******************************************************************************/

/******************************************************************************/
/* Local type definitions ('typedef')                                         */
/******************************************************************************/

/******************************************************************************/
/* Local function prototypes ('static')                                       */
/******************************************************************************/

/******************************************************************************/
/* Local variable definitions ('static')                                      */
/******************************************************************************/
static uint32_t u32CountDt0;
static uint32_t u32CountDt1;

// Configuration for channel 0
static const stc_dt_channel_config_t stcDtChannelConfig0 = {
    DtPeriodic,         // Periodic mode
    DtPrescalerDiv256,  // Prescaler dividor f/256
    DtCounterSize32     // 32bits counter size
};

// Configuration for channel 1
static const stc_dt_channel_config_t stcDtChannelConfig1 = {
    DtOneShot,          // One-shot mode
    DtPrescalerDiv256,  // Prescaler dividor f/256
    DtCounterSize32     // 32bits counter size
};

/******************************************************************************/
/* Local pre-processor symbols/macros ('#define')                             */
/******************************************************************************/

/******************************************************************************/
/* Function implementation - global ('extern') and local ('static')           */
/******************************************************************************/
/**
 ******************************************************************************
 ** \brief Initializatio GPIO
 **
 ******************************************************************************/
static void PortInit(void)
{
    Gpio1pin_InitOut(GPIO1PIN_P61, Gpio1pin_InitVal(1u));
}

/**
 ******************************************************************************
 ** \brief Set port for LED1
 **
 ******************************************************************************/
static void SetLed0(uint32_t u32Led0)
{
    if (0 == (u32Led0 & 0x00000001))
    {
       Gpio1pin_Put(GPIO1PIN_P61, 0u);
    }
    else
    {
       Gpio1pin_Put(GPIO1PIN_P61, 1u);
    }
}

/**
 ******************************************************************************
 ** \brief Callback Function for DT channel 0
 **
 ******************************************************************************/
static void Dt0Callback(void)
{
    Dt_ClrIntFlag(DtChannel0);    // Clear Irq
    u32CountDt0++;
    SetLed0(u32CountDt0);
}

/**
 ******************************************************************************
 ** \brief Callback Function for DT channel 1
 **
 ******************************************************************************/
static void Dt1Callback(void)
{
    Dt_ClrIntFlag(DtChannel1);    // Clear Irq
    u32CountDt1++;
    // Channel 1 repeats 9 times.
    if (u32CountDt1 < 8)
    {
        Dt_WriteLoadVal(312500, DtChannel1);
    }
}

/**
 ******************************************************************************
 ** \brief  Main function of project for SE61A1 series (FM0P).
 **
 ** \return uint32_t return value, if needed
 **
 ** This sample uses dual timer 2 channels. Mode of each channel is as follows.
 ** Channel 0 : Interval mode
 ** Channel 1 : One-shot mode
 ** Interval time for channel 0 is 0.5 sec at first, then interval time changes
 ** to 1 sec.
 ** Overflow time for channel 1 is 1sec. This channel repeats 9 times.
 **
 ******************************************************************************/
int32_t main(void)
{
    PortInit();

    // Initialize dual timer channel 0
    if (Ok != Dt_Init((stc_dt_channel_config_t*)&stcDtChannelConfig0, DtChannel0))
    {
#ifdef DEBUG_PRINT
        printf("Initial error channel0!\n");
#endif
        while(1);
    }

    // Initialize dual timer channel 1
    if (Ok != Dt_Init((stc_dt_channel_config_t*)&stcDtChannelConfig1, DtChannel1))
    {
#ifdef DEBUG_PRINT
        printf("Initial error channel1!\n");
#endif
        while(1);
    }

    // Initialize interrupt counter
    u32CountDt0 = 0;
    u32CountDt1 = 0;

    // Write load value for channel 0 (0.5sec interval)
    Dt_WriteLoadVal(78125, DtChannel0);
    // Write background load value for channel 0 (0.5sec -> 1sec)
    Dt_WriteBgLoadVal(156250, DtChannel0);
    // Enable interrupt for channel 0
    Dt_EnableInt(Dt0Callback, DtChannel0);
    // Start count for channel 0
    Dt_EnableCount(DtChannel0);

    // Write load value for channel 1 (1sec until overflow)
    Dt_WriteLoadVal(156250, DtChannel1);
    // Enable interrupt for channel 1
    Dt_EnableInt(Dt1Callback, DtChannel1);
    // Start count for channel 1
    Dt_EnableCount(DtChannel1);

    while(1);
}

/******************************************************************************/
/* EOF (not truncated)                                                        */
/******************************************************************************/
