/*******************************************************************************
* Copyright (C) 2013 Spansion LLC. All Rights Reserved. 
*
* This software is owned and published by: 
* Spansion LLC, 915 DeGuigne Dr. Sunnyvale, CA  94088-3453 ("Spansion").
*
* BY DOWNLOADING, INSTALLING OR USING THIS SOFTWARE, YOU AGREE TO BE BOUND 
* BY ALL THE TERMS AND CONDITIONS OF THIS AGREEMENT.
*
* This software contains source code for use with Spansion 
* components. This software is licensed by Spansion to be adapted only 
* for use in systems utilizing Spansion components. Spansion shall not be 
* responsible for misuse or illegal use of this software for devices not 
* supported herein.  Spansion is providing this software "AS IS" and will 
* not be responsible for issues arising from incorrect user implementation 
* of the software.  
*
* SPANSION MAKES NO WARRANTY, EXPRESS OR IMPLIED, ARISING BY LAW OR OTHERWISE,
* REGARDING THE SOFTWARE (INCLUDING ANY ACOOMPANYING WRITTEN MATERIALS), 
* ITS PERFORMANCE OR SUITABILITY FOR YOUR INTENDED USE, INCLUDING, 
* WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY, THE IMPLIED 
* WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE OR USE, AND THE IMPLIED 
* WARRANTY OF NONINFRINGEMENT.  
* SPANSION SHALL HAVE NO LIABILITY (WHETHER IN CONTRACT, WARRANTY, TORT, 
* NEGLIGENCE OR OTHERWISE) FOR ANY DAMAGES WHATSOEVER (INCLUDING, WITHOUT 
* LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS, BUSINESS INTERRUPTION, 
* LOSS OF BUSINESS INFORMATION, OR OTHER PECUNIARY LOSS) ARISING FROM USE OR 
* INABILITY TO USE THE SOFTWARE, INCLUDING, WITHOUT LIMITATION, ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL OR CONSEQUENTIAL DAMAGES OR LOSS OF DATA, 
* SAVINGS OR PROFITS, 
* EVEN IF SPANSION HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. 
* YOU ASSUME ALL RESPONSIBILITIES FOR SELECTION OF THE SOFTWARE TO ACHIEVE YOUR
* INTENDED RESULTS, AND FOR THE INSTALLATION OF, USE OF, AND RESULTS OBTAINED 
* FROM, THE SOFTWARE.  
*
* This software may be replicated in part or whole for the licensed use, 
* with the restriction that this Disclaimer and Copyright notice must be 
* included with each copy of this software, whether used in part or whole, 
* at all times.  
*/
/******************************************************************************/
/** \file main.c
 **
 ** This example demonstrates I2C data transfer and receive in master mode. 
 ** This example must be used with the example "i2c_slave_polling".
 ** Make following connection before using this example:
 ** Master (1st board "i2c_master_polling")   Slave (2nd board with "i2c_slave_polling")
 ** SCK0_0   ---------------------------     SCK1_1
 ** SOT0_0   ---------------------------     SOT1_1
 **
 ** History:
 **   - 2014-02-18  1.0  Edison Zhang         First version.
 **
 ******************************************************************************/

/******************************************************************************/
/* Include files                                                              */
/******************************************************************************/
#include "pdl_header.h"

/*****************************************************************************/
/* Local pre-processor symbols/macros ('#define')                            */
/*****************************************************************************/
/*****************************************************************************/
/* Global variable definitions (declared in header file with 'extern')       */
/*****************************************************************************/
/*****************************************************************************/
/* Local type definitions ('typedef')                                        */
/*****************************************************************************/
#define SAMPLE_I2C_MASTER_TX_BUFFSIZE   sizeof(au8I2cMasterTxBuf)/sizeof(char)
#define SAMPLE_I2C_MASTER_RX_BUFFSIZE   SAMPLE_I2C_MASTER_TX_BUFFSIZE

#define I2C_RET_OK                  0
#define I2C_RET_ERROR               1

#define I2C_DEV_ADDR              (0x3Au)
#define I2C_DEV_ADDR_W            (0x3Au | 0u)
#define I2C_DEV_ADDR_R            (0x3Au | 1u)

/*****************************************************************************/
/* Local function prototypes ('static')                                      */
/*****************************************************************************/
/*****************************************************************************/
/* Local variable definitions ('static')                                     */
/*****************************************************************************/
volatile stc_mfsn_i2c_t* I2cCh0 = &I2C0;
static uint8_t au8I2cMasterTxBuf[10] = "0123456789";
static uint8_t au8I2cMasterRxBuf[SAMPLE_I2C_MASTER_RX_BUFFSIZE];

/**
 ******************************************************************************
 ** \brief  Compare each data in the input two buffers
 **
 ** \relval Ok     The data in buffer1 are same with that in buffer2
 ** \relval Error  The data in buffer1 are not same with that in buffer2 
 ******************************************************************************/
static en_result_t CompareData(uint8_t* pBuf1, uint8_t* pBuf2, uint8_t u8Length)
{
    while(u8Length--)
    {
        if(*pBuf1++ != *pBuf2++)
        {
            return Error;
        }
    }
    
    return Ok;
}

/**
 ******************************************************************************
 ** \brief  Initialize I2C Master
 ******************************************************************************/
static void InitI2cMaster(void)
{
    stc_mfs_i2c_config_t stcI2c0Config;
        
    SetPinFunc_SOT0_0();
    SetPinFunc_SCK0_0();
    
    stcI2c0Config.enMsMode = I2cMaster;
    stcI2c0Config.u32BaudRate = 100000u;
    stcI2c0Config.bWaitSelection = FALSE;
    stcI2c0Config.bDmaEnable = FALSE;
    stcI2c0Config.bEnableFifo = FALSE;
    
    Mfs_I2c_Init(I2cCh0, &stcI2c0Config);

}

/*!
 ******************************************************************************
 ** \brief  Config the start condition
 **
 ** \param  none
 **
 ** \return none
 ******************************************************************************
 */
static uint8_t I2c_Start(uint8_t Addr)
{
    /* Prepare I2C device address */
    Mfs_I2c_SendData(I2cCh0, Addr);
    /* Generate I2C start signal */
    if(Ok != Mfs_I2c_GenerateStart(I2cCh0))
    {
        return I2C_RET_ERROR; /* Timeout or other error */
    }

    while(1)
    {
        if(TRUE != Mfs_I2c_GetStatus(I2cCh0, I2cRxTxInt))
        {
            break;
        }
    }
   
    if(I2cNAck == Mfs_I2c_GetAck(I2cCh0))
    {
        return I2C_RET_ERROR;   /* NACK */
    }
    
    if(TRUE == Mfs_I2c_GetStatus(I2cCh0, I2cBusErr))
    {
        return I2C_RET_ERROR; /* Bus error occurs? */
    }
    
    if(TRUE == Mfs_I2c_GetStatus(I2cCh0, I2cOverrunError))
    {
        return I2C_RET_ERROR; /* Overrun error occurs? */
    }
   
    return I2C_RET_OK;
}
/*!
 ******************************************************************************
 ** \brief  Transmit the dedicated size data to device
 **
 ** \param  pTxData Transmit data pointer
 **
 ** \param  u8Size the number of transmitted data
 **
 ** \return none
 ******************************************************************************
 */
static uint8_t I2c_Write(uint8_t *pTxData, uint8_t u8Size)
{
    uint8_t i;
    
    for(i=0;i<u8Size;i++)
    {
        /* Transmit the data */
        Mfs_I2c_SendData(I2cCh0, pTxData[i]);
        Mfs_I2c_ClrStatus(I2cCh0, I2cRxTxInt);
        /* Wait for end of transmission */
        while(1)
        {
            if(TRUE == Mfs_I2c_GetStatus(I2cCh0, I2cRxTxInt))
            {
                break;
            }
        }
        
        while(1)
        {
            if(TRUE == Mfs_I2c_GetStatus(I2cCh0, I2cTxEmpty))
            {
                break;
            }
        }
       
        if(I2cNAck == Mfs_I2c_GetAck(I2cCh0))
        {
            return I2C_RET_ERROR;   /* NACK */
        }
        
        if(TRUE == Mfs_I2c_GetStatus(I2cCh0, I2cBusErr))
        {
            return I2C_RET_ERROR; /* Bus error occurs? */
        }
        
        if(TRUE == Mfs_I2c_GetStatus(I2cCh0, I2cOverrunError))
        {
            return I2C_RET_ERROR; /* Overrun error occurs? */
        }
    }
    
    return I2C_RET_OK;
}
/*!
 ******************************************************************************
 ** \brief  Receive the dedicated size data to device
 **
 ** \param  pRxData The receive buffer pointer
 **
 ** \param  u8Size The number of receiving data
 **
 ** \return none
 ******************************************************************************
 */
static uint8_t I2c_Read(uint8_t *pRxData, uint8_t u8Size)
{
    uint8_t i = 0;
    /* Clear interrupt flag generated by device address send */
    Mfs_I2c_ClrStatus(I2cCh0, I2cRxTxInt);
    
    if(I2cNAck == Mfs_I2c_GetAck(I2cCh0))
    {
        return I2C_RET_ERROR;   /* NACK */
    }
    
    while(i < u8Size)
    {   
        /* Wait for the receive data */
        while(1)
        {
            if(TRUE == Mfs_I2c_GetStatus(I2cCh0, I2cRxTxInt))
            {
                break;
            }
        }
        
        if(i == u8Size-1)
        {
            Mfs_I2c_ConfigAck(I2cCh0, I2cNAck); /* Last byte send a NACK */
        }
        else
        {
            Mfs_I2c_ConfigAck(I2cCh0, I2cAck);
        }
        
        /* Clear interrupt flag and receive data to RX buffer */
        Mfs_I2c_ClrStatus(I2cCh0, I2cRxTxInt);
        
        /* Wait for the receive data */
        while(1)
        {
            if(TRUE == Mfs_I2c_GetStatus(I2cCh0, I2cRxFull))
            {
                break;
            }
        }
         
        if(TRUE == Mfs_I2c_GetStatus(I2cCh0, I2cBusErr))
        {
            return I2C_RET_ERROR;   /* Bus error occurs? */
        }
        
        if(TRUE == Mfs_I2c_GetStatus(I2cCh0, I2cOverrunError))
        {
            return I2C_RET_ERROR;  /* Overrun error occurs? */
        }
        
        pRxData[i++] = Mfs_I2c_ReceiveData(I2cCh0);
    }
    return I2C_RET_OK;
}
/*!
 ******************************************************************************
 ** \brief  Config the stop condition
 **
 ** \param  none
 **
 ** \return none
 ******************************************************************************
 */
static uint8_t I2c_Stop(void)
{
    /* Generate I2C start signal */
    if(Ok != Mfs_I2c_GenerateStop(I2cCh0))
    {
        return I2C_RET_ERROR; /* Timeout or other error */
    }
    /* Clear Stop condition flag */
    while(1)
    {
        if(TRUE == Mfs_I2c_GetStatus(I2cCh0, I2cStopDetect))
        {
            break;
        }
    }
   
    if(TRUE == Mfs_I2c_GetStatus(I2cCh0, I2cBusErr))
    {
        return I2C_RET_ERROR;
    }
   
    Mfs_I2c_ClrStatus(I2cCh0, I2cStopDetect);
    Mfs_I2c_ClrStatus(I2cCh0, I2cRxTxInt);
    
    return I2C_RET_OK;
}

/**
 ******************************************************************************
 ** \brief  Main function of project for S6E1A1 series (FM0P).
 **
 ** \return uint32_t return value, if needed
 ******************************************************************************/
int32_t main(void)
{     
    InitI2cMaster();
    
    /* I2C Master Write data */
    I2c_Start(I2C_DEV_ADDR_W);
    I2c_Write(au8I2cMasterTxBuf, SAMPLE_I2C_MASTER_TX_BUFFSIZE);
    I2c_Stop();
    
    /* I2C Master Read data */
    I2c_Start(I2C_DEV_ADDR_R);
    I2c_Read(au8I2cMasterRxBuf, SAMPLE_I2C_MASTER_RX_BUFFSIZE);
    I2c_Stop();
            
    if(Ok != CompareData(au8I2cMasterTxBuf, au8I2cMasterRxBuf, SAMPLE_I2C_MASTER_TX_BUFFSIZE))
    {
        while(1); /* Communication fails if code runs here. */
    }
    
    while(1); /* Data is normally sent and received */
}


/******************************************************************************/
/* EOF (not truncated)                                                        */
/******************************************************************************/
