/*******************************************************************************
* Copyright (C) 2013 Spansion LLC. All Rights Reserved. 
*
* This software is owned and published by: 
* Spansion LLC, 915 DeGuigne Dr. Sunnyvale, CA  94088-3453 ("Spansion").
*
* BY DOWNLOADING, INSTALLING OR USING THIS SOFTWARE, YOU AGREE TO BE BOUND 
* BY ALL THE TERMS AND CONDITIONS OF THIS AGREEMENT.
*
* This software contains source code for use with Spansion 
* components. This software is licensed by Spansion to be adapted only 
* for use in systems utilizing Spansion components. Spansion shall not be 
* responsible for misuse or illegal use of this software for devices not 
* supported herein.  Spansion is providing this software "AS IS" and will 
* not be responsible for issues arising from incorrect user implementation 
* of the software.  
*
* SPANSION MAKES NO WARRANTY, EXPRESS OR IMPLIED, ARISING BY LAW OR OTHERWISE,
* REGARDING THE SOFTWARE (INCLUDING ANY ACOOMPANYING WRITTEN MATERIALS), 
* ITS PERFORMANCE OR SUITABILITY FOR YOUR INTENDED USE, INCLUDING, 
* WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY, THE IMPLIED 
* WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE OR USE, AND THE IMPLIED 
* WARRANTY OF NONINFRINGEMENT.  
* SPANSION SHALL HAVE NO LIABILITY (WHETHER IN CONTRACT, WARRANTY, TORT, 
* NEGLIGENCE OR OTHERWISE) FOR ANY DAMAGES WHATSOEVER (INCLUDING, WITHOUT 
* LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS, BUSINESS INTERRUPTION, 
* LOSS OF BUSINESS INFORMATION, OR OTHER PECUNIARY LOSS) ARISING FROM USE OR 
* INABILITY TO USE THE SOFTWARE, INCLUDING, WITHOUT LIMITATION, ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL OR CONSEQUENTIAL DAMAGES OR LOSS OF DATA, 
* SAVINGS OR PROFITS, 
* EVEN IF SPANSION HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. 
* YOU ASSUME ALL RESPONSIBILITIES FOR SELECTION OF THE SOFTWARE TO ACHIEVE YOUR
* INTENDED RESULTS, AND FOR THE INSTALLATION OF, USE OF, AND RESULTS OBTAINED 
* FROM, THE SOFTWARE.  
*
* This software may be replicated in part or whole for the licensed use, 
* with the restriction that this Disclaimer and Copyright notice must be 
* included with each copy of this software, whether used in part or whole, 
* at all times.  
*/
/******************************************************************************/
/** \file main.c
 **
 ** This example demonstrates LIN slave mode with interrupt mode, 
 ** which must be used with LIN master sample ("lin_master_polling")
 ** A LIN transceiver must be used and make following connection before starting 
 ** this example:
 ** ----------------------------------------------------------------------------
 ** LIN Slave                                                         LIN master
 **          TX  ------------------           ------------------   TX
 ** SIN1_1-------|                 | LIN Bus |                  |--------SIN1_1
 **          RX  | LIN transceiver |---------|   LIN transceiver|  RX 
 ** SOT1_1-------|                 |         |                  |--------SOT1_1
 **              ------------------          -------------------
 ** 
 ** History:
 **   - 2014-03-14  1.0  Edison Zhang         First version.
 **
 ******************************************************************************/

/******************************************************************************/
/* Include files                                                              */
/******************************************************************************/
#include "pdl_header.h"
#include <string.h>

/*****************************************************************************/
/* Local pre-processor symbols/macros ('#define')                            */
/*****************************************************************************/
#define LIN_SYNC_ICU              (1u) //LIN ch.1's SYNC connected with ICU1

#define LIN_M_TX_S_RX_ID           0x30
#define LIN_M_RX_S_TX_ID           0x31

#define LinCh1                     &LIN1
#define InitLinIo()                {SetPinFunc_SIN1_1();SetPinFunc_SOT1_1();}
/*****************************************************************************/
/* Global variable definitions (declared in header file with 'extern')       */
/*****************************************************************************/
/*****************************************************************************/
/* Local type definitions ('typedef')                                        */
/*****************************************************************************/
/**
 ******************************************************************************
 ** \brief Lin sequence
 ******************************************************************************/
typedef enum en_lin_seq
{
    LinSeqBreak,                 ///< Lin break filed handler process
    LinSeqSync,                  ///< Lin sync filed handler process
    LinSeqId,                    ///< Lin ID handler process
    LinSeqData,                  ///< Lin data handler process
    LinSeqChecksum,              ///< Lin checksum handler process
    LinSeqEnd,                   ///< Lin frame transfer end
    
}en_lin_seq_t;

/**
 ******************************************************************************
 ** \brief Lin mode
 ******************************************************************************/
typedef enum en_lin_mode
{
    LinTxMode,          ///< TX mode
    LinRxMode,          ///< RX mode
    
}en_lin_mode_t;

/**
 ******************************************************************************
 ** \brief Lin information structure
 ******************************************************************************/
typedef struct lin_info
{
    en_lin_seq_t enSeq;        ///< Lin sequence
    uint8_t u8Id;              ///< Lin frame ID
    en_lin_mode_t enMode;      ///< Lin mode
    uint8_t* pu8TxData;        ///< Pointer to TX buffer
    uint8_t  u8TxSize;         ///< TX size
    uint8_t* pu8RxData;        ///< Pointer to RX buffer
    uint8_t  u8RxSize;         ///< RX size
    uint8_t  u8Cnt;            ///< Data transfer count
    boolean_t bErrorFlag;      ///< Error flag
    uint8_t  u8Checksum;       ///< Lin frame check sum
    boolean_t bRwSyncFlag;     ///< Lin transfer and receive sync flag (in TX mode)
    
}lin_info_t;

/**
 ******************************************************************************
 ** \brief Lin sync field handler structure
 ******************************************************************************/
typedef struct lin_sync_info
{
    uint32_t u32a;         ///< 1st value
    uint32_t u32b;         ///< 2nd value
    uint32_t u32Pclk;      ///< MFS PCLK
    uint32_t u32bgr;       ///< BGR value
    uint32_t u32Baudrate;  ///< LIN baud rate
    uint8_t u8Cnt;         ///< Capture times count
    uint8_t u8Overflow;    ///< Overflow flag
    
}lin_sync_info_t;

/*****************************************************************************/
/* Local function prototypes ('static')                                      */
/*****************************************************************************/
static uint8_t LinCalChecksum(uint8_t *pu8Data, uint8_t u8Len);
static void LinTxIntCallback(void);
static void LinRxIntCallback(void);
static void FrtPeakMatchIntCallback(void);
static void IcuIntCallback(void);
static void LinBreakIntCallback(void);

/*****************************************************************************/
/* Local variable definitions ('static')                                     */
/*****************************************************************************/
static uint8_t au8LinTxBuf[8];
static uint8_t au8LinRxBuf[8];
static lin_info_t stcLinInfo;
static lin_sync_info_t stcLinSyncInfo;
static stc_lin_int_sel_t stcLinIntSel;
static stc_lin_int_cb_t  stcLinIntCb;
static stc_frt_int_sel_t stcFrtIntSel;
static stc_frt_int_cb_t  stcFrtIntCb;

/**
 ******************************************************************************
 ** \brief Calculate LIN data checksum
 **
 ** \param pu8Data Poitner to data buffer
 ** \param u8Len   Data length
 **
 ** \relval Data checksum
 ******************************************************************************/
static uint8_t LinCalChecksum(uint8_t *pu8Data, uint8_t u8Len)
{
    uint16_t u16Sum = 0;
    uint8_t u8i;
    for(u8i = 0; u8i < u8Len; u8i++)
    {
        u16Sum += pu8Data[u8i];
        if(u16Sum & 0xFF00)
        {
            u16Sum = (u16Sum & 0x00FF) + 1;
        }
    }
    u16Sum ^= 0x00FF;
    return (uint8_t)u16Sum;
}

/**
 ******************************************************************************
 ** \brief LIN TX interrupt callback function
 ******************************************************************************/
static void LinTxIntCallback(void)
{
    if(stcLinInfo.bRwSyncFlag != TRUE)
    {
        return;
    }
  
    switch (stcLinInfo.enSeq)
    {
        case LinSeqData:
            if(stcLinInfo.enMode == LinTxMode)
            {
                Mfs_Lin_SendData(LinCh1, *stcLinInfo.pu8TxData);
                stcLinInfo.bRwSyncFlag = FALSE;
            }
            break;
        case LinSeqChecksum:
            if(stcLinInfo.enMode == LinTxMode)
            {
                Mfs_Lin_SendData(LinCh1, stcLinInfo.u8Checksum);  
                stcLinInfo.bRwSyncFlag = FALSE;
                
                PDL_ZERO_STRUCT(stcLinIntSel);
                stcLinIntSel.bTxInt = TRUE;
                Mfs_Lin_DisableInt(LinCh1, &stcLinIntSel);
            }
            break;
        default:
            break;
    }
    
}

/**
 ******************************************************************************
 ** \brief LIN RX interrupt callback function
 ******************************************************************************/
static void LinRxIntCallback(void)
{
    uint8_t u8RdData;

    if((TRUE == Mfs_Lin_GetStatus(LinCh1, LinFrameError)) || 
       (TRUE == Mfs_Lin_GetStatus(LinCh1, LinOverrunError)))
    {
        PDL_ZERO_STRUCT(stcLinIntSel);
        stcLinInfo.bErrorFlag = 1;
        stcLinIntSel.bTxInt = TRUE;
        stcLinIntSel.bRxInt = TRUE;
        Mfs_Lin_DisableInt(LinCh1, &stcLinIntSel);
        return;
    }
  
    u8RdData = Mfs_Lin_ReceiveData(LinCh1);
    
    switch (stcLinInfo.enSeq)
    {
        case LinSeqId:
            if(u8RdData != stcLinInfo.u8Id)
            {
                stcLinInfo.bErrorFlag = 1;
                PDL_ZERO_STRUCT(stcLinIntSel);
                stcLinIntSel.bTxInt = TRUE;
                stcLinIntSel.bRxInt = TRUE;
                Mfs_Lin_DisableInt(LinCh1, &stcLinIntSel);
                return;
            }
            stcLinInfo.enSeq++;
            
            if(stcLinInfo.enMode == LinTxMode)
            {
                PDL_ZERO_STRUCT(stcLinIntSel);
                PDL_ZERO_STRUCT(stcLinIntCb);
                stcLinIntSel.bTxInt = TRUE;
                stcLinIntCb.pfnTxIntCb = LinTxIntCallback;
                Mfs_Lin_EnableInt(LinCh1, &stcLinIntSel, &stcLinIntCb);  
                
                /* Enable TX function of LIN   */
                Mfs_Lin_EnableFunc(LinCh1, LinTx);
                
                stcLinInfo.bRwSyncFlag = TRUE;
            }
            
            break;
        case LinSeqData:
            if(stcLinInfo.enMode == LinTxMode)
            {
                if(u8RdData != *stcLinInfo.pu8TxData)
                {
                    stcLinInfo.bErrorFlag = 1;
                    PDL_ZERO_STRUCT(stcLinIntSel);
                    stcLinIntSel.bTxInt = TRUE;
                    stcLinIntSel.bRxInt = TRUE;
                    Mfs_Lin_DisableInt(LinCh1, &stcLinIntSel);
                }
                
                stcLinInfo.u8Cnt++;
                stcLinInfo.pu8TxData++;
                stcLinInfo.bRwSyncFlag = TRUE;
                
                if(stcLinInfo.u8Cnt == stcLinInfo.u8TxSize)
                {
                    stcLinInfo.enSeq++;
                }
            }
            else
            {
                *stcLinInfo.pu8RxData = u8RdData;
                stcLinInfo.pu8RxData++;
                stcLinInfo.u8Cnt++;
                
                if(stcLinInfo.u8Cnt == stcLinInfo.u8RxSize)
                {
                    stcLinInfo.enSeq++;
                }
            }
            break;
        case LinSeqChecksum:
            if(stcLinInfo.enMode == LinTxMode)
            {
                if(u8RdData != stcLinInfo.u8Checksum)
                {
                    stcLinInfo.bErrorFlag = 1;
                    PDL_ZERO_STRUCT(stcLinIntSel);
                    stcLinIntSel.bTxInt = TRUE;
                    stcLinIntSel.bRxInt = TRUE;
                    Mfs_Lin_DisableInt(LinCh1, &stcLinIntSel);
                    return;
                }
                
                PDL_ZERO_STRUCT(stcLinIntSel);
                stcLinIntSel.bTxInt = TRUE;
                stcLinIntSel.bRxInt = TRUE;
                Mfs_Lin_DisableInt(LinCh1, &stcLinIntSel);
                
                stcLinInfo.enSeq = LinSeqEnd;
            }
            else
            {
                stcLinInfo.u8Checksum = LinCalChecksum((stcLinInfo.pu8RxData-stcLinInfo.u8RxSize), stcLinInfo.u8RxSize);
                if(stcLinInfo.u8Checksum  != u8RdData)
                {
                    stcLinInfo.bErrorFlag = 1;
                }
                else
                {
                    stcLinInfo.enSeq = LinSeqEnd;
                }
                
                stcLinIntSel.bRxInt = TRUE;
                Mfs_Lin_DisableInt(LinCh1, &stcLinIntSel);
                break;
            }
            break;
        default:
            break;
    }
    
}

/**
 ******************************************************************************
 ** \brief FRT peak match interrupt callback function
 ******************************************************************************/
static void FrtPeakMatchIntCallback(void)
{
    stcLinSyncInfo.u8Overflow++;
}

/**
 ******************************************************************************
 ** \brief ICU interrupt callback function
 ******************************************************************************/
static void IcuIntCallback(void)
{
    if(stcLinSyncInfo.u8Cnt == 0u)
    {
        stcLinSyncInfo.u32a = Mft_Icu_GetCaptureData(&MFT0_ICU, LIN_SYNC_ICU);
        stcLinSyncInfo.u8Cnt++;
    }
    else if(stcLinSyncInfo.u8Cnt == 1u)
    {
        stcLinSyncInfo.u32b = Mft_Icu_GetCaptureData(&MFT0_ICU, LIN_SYNC_ICU);                
        stcLinSyncInfo.u32bgr = (50000*stcLinSyncInfo.u8Overflow + stcLinSyncInfo.u32b - stcLinSyncInfo.u32a)/8-1;
        
        Mft_Frt_Stop(&MFT0_FRT, 0u);
        Mft_Icu_ConfigDetectMode(&MFT0_ICU, LIN_SYNC_ICU, IcuDisable);
                
        stcLinSyncInfo.u32Pclk = __HCLK/(1ul << (APBC1_PSR_Val & 0x03u)); /* MFS is attached on APB1 bus */
        stcLinSyncInfo.u32Baudrate = stcLinSyncInfo.u32Pclk/(stcLinSyncInfo.u32bgr+1);
        Mfs_Lin_SetBaudRate(LinCh1, stcLinSyncInfo.u32Baudrate);
        
        /* Configure interrupt */
        PDL_ZERO_STRUCT(stcLinIntSel);
        PDL_ZERO_STRUCT(stcLinIntCb);
      
        stcLinIntSel.bRxInt = TRUE;
        stcLinIntCb.pfnRxIntCb = LinRxIntCallback;
        
        Mfs_Lin_EnableInt(LinCh1, &stcLinIntSel, &stcLinIntCb);
        
        /* Enable RX function of LIN   */
        Mfs_Lin_EnableFunc(LinCh1, LinRx);
        
        stcLinInfo.enSeq++;
    }
}

/**
 ******************************************************************************
 ** \brief LIN break field interrupt callback interrupt
 ******************************************************************************/
static void LinBreakIntCallback(void)
{
    /* Capture sync field and calculate baudrate */
    Mft_Icu_ConfigDetectMode(&MFT0_ICU, LIN_SYNC_ICU, IcuBothDetect);
    Mft_Frt_Start(&MFT0_FRT, 0u);
    
    PDL_ZERO_STRUCT(stcLinSyncInfo);
    
    stcLinInfo.enSeq++;
}


/**
 ******************************************************************************
 ** \brief Initialize LIN, FRT, ICU hardware
 ******************************************************************************/
void InitLinSlave(void)
{
    stc_mfs_lin_config_t stcLinConfig;
    stc_mft_frt_config_t stcFrtConfig;
    
    /* Initialize LIN function I/O */
    InitLinIo();
 
    /* Initialize LIN  */
    stcLinConfig.enMsMode = LinSlaveMode;
    stcLinConfig.u32BaudRate = 10000;   /* Set an initial value */
    stcLinConfig.enStopBits = LinOneStopBit;
    stcLinConfig.bEnableFifo = FALSE;
    
    Mfs_Lin_Init(LinCh1, &stcLinConfig);
    
    /* Initialize FRT */
    stcFrtConfig.enFrtMode = FrtUpCount;
    stcFrtConfig.enFrtClockDiv = FrtPclkDiv1;
    stcFrtConfig.bEnExtClock = FALSE;
    stcFrtConfig.bEnBuffer = TRUE;

    Mft_Frt_Stop(&MFT0_FRT ,0u);
    Mft_Frt_Init(&MFT0_FRT ,0u, &stcFrtConfig);
    Mft_Frt_SetCountCycle(&MFT0_FRT, 0u, 50000);
    
    /* Initialize ICU */
    FM0P_GPIO->EPFR01_f.IC01S = 4u; // select LIN ch.1 LSYN to IC01
    
    Mft_Icu_SelFrt(&MFT0_ICU, LIN_SYNC_ICU, Frt0ToIcu);
    Mft_Icu_ConfigDetectMode(&MFT0_ICU, LIN_SYNC_ICU, IcuDisable);
    
    /* Configure interrupt */
    PDL_ZERO_STRUCT(stcFrtIntSel);
    PDL_ZERO_STRUCT(stcFrtIntCb);
  
    stcFrtIntSel.bFrtPeakMatchInt = TRUE;
    stcFrtIntCb.pfnFrtPeakCallback = FrtPeakMatchIntCallback;
    
    Mft_Frt_EnableInt(&MFT0_FRT, 0u, &stcFrtIntSel, &stcFrtIntCb);
    
    Mft_Icu_EnableInt(&MFT0_ICU, LIN_SYNC_ICU, IcuIntCallback);
    
}

/**
 ******************************************************************************
 ** \brief Transfer a LIN frame
 **
 ** \param u8Id    LIN frame ID
 ** \param pu8Data Poitner to data buffer
 ** \param u32Size Data size
 **
 ** \relval Ok Frame transfer OK
 ** \relval Error Frame transfer failed
 ******************************************************************************/
en_result_t LinSlaveTx(uint8_t u8Id, uint8_t *pu8Data, uint32_t u32Size)
{
    stcLinInfo.enSeq = LinSeqBreak;
    stcLinInfo.u8Id = u8Id;
    stcLinInfo.enMode = LinTxMode;
    stcLinInfo.pu8TxData = pu8Data;
    stcLinInfo.u8TxSize = u32Size;
    stcLinInfo.u8Cnt = 0;
    stcLinInfo.u8Checksum = LinCalChecksum(pu8Data, u32Size);
    
    /* Configure interrupt */
    PDL_ZERO_STRUCT(stcLinIntSel);
    PDL_ZERO_STRUCT(stcLinIntCb);
  
    stcLinIntSel.bLinBreakInt = TRUE;
    stcLinIntCb.pfnLinBreakIntCb = LinBreakIntCallback;
    
    Mfs_Lin_EnableInt(LinCh1, &stcLinIntSel, &stcLinIntCb);
    
    while(stcLinInfo.enSeq < LinSeqEnd)
    {
        if(stcLinInfo.bErrorFlag == 1)
        {
             /* Disable RX function of LIN   */
            Mfs_Lin_DisableFunc(LinCh1, LinTx);
            Mfs_Lin_DisableFunc(LinCh1, LinRx);
            return Error;
        }
    }
    
     /* Disable RX function of LIN   */
    Mfs_Lin_DisableFunc(LinCh1, LinTx);
    Mfs_Lin_DisableFunc(LinCh1, LinRx);
    
    return Ok;
}

/**
 ******************************************************************************
 ** \brief Receive a LIN frame
 **
 ** \param u8Id    LIN frame ID
 ** \param pu8Data Poitner to data buffer
 ** \param u32Size Data size
 **
 ** \relval Ok Frame receive OK
 ** \relval Error Frame receive failed
 ******************************************************************************/
en_result_t LinSlaveRx(uint8_t u8Id, uint8_t *pu8Data, uint32_t u32Size)
{
    stcLinInfo.u8Id = u8Id;
    stcLinInfo.u8Cnt = 0;
    stcLinInfo.pu8RxData = pu8Data;
    stcLinInfo.u8RxSize = u32Size;
    stcLinInfo.enMode = LinRxMode;
    stcLinInfo.enSeq = LinSeqBreak;
    
    /* Configure interrupt */
    PDL_ZERO_STRUCT(stcLinIntSel);
    PDL_ZERO_STRUCT(stcLinIntCb);
  
    stcLinIntSel.bLinBreakInt = TRUE;
    stcLinIntCb.pfnLinBreakIntCb = LinBreakIntCallback;
    
    Mfs_Lin_EnableInt(LinCh1, &stcLinIntSel, &stcLinIntCb);
    
    while(stcLinInfo.enSeq < LinSeqEnd)
    {
        if(stcLinInfo.bErrorFlag == 1)
        {
             /* Enable RX function of LIN   */
            Mfs_Lin_DisableFunc(LinCh1, LinRx);
            return Error;
        }
    }
    
     /* Disable RX function of LIN   */
    Mfs_Lin_DisableFunc(LinCh1, LinRx);
    
    return Ok;
}

/**
 ******************************************************************************
 ** \brief  Main function of project for S6E1A1 series (FM0P).
 **
 ** \return uint32_t return value, if needed
 ******************************************************************************/
int32_t main(void)
{   
    InitLinSlave();
     
    while(1)
    {
        if(Ok != LinSlaveRx(LIN_M_TX_S_RX_ID, au8LinRxBuf, sizeof(au8LinRxBuf)))
        {
            while(1);
        }
        
        memcpy(au8LinTxBuf, au8LinRxBuf, sizeof(au8LinRxBuf));
        
        if(Ok != LinSlaveTx(LIN_M_RX_S_TX_ID, au8LinTxBuf, sizeof(au8LinTxBuf)))
        {
            while(1);
        }
    }
}


/******************************************************************************/
/* EOF (not truncated)                                                        */
/******************************************************************************/
