/************************************************************************/
/*   (C) FUJITSU SEMICONDUCTOR LIMITED AND ITS SUBSIDIARIES (FSLGROUP)  */
/*                                                                      */
/* The following software deliverable is subject the readme text        */
/* included in the same directory or parent directory of this software  */
/* and the use agreement as found on FSLGROUP's Website                 */
/* www.fujitsu.com/global/services/microelectronics/product/micom/      */
/* regarding this software.                                             */
/* If you do not agree with such terms then DO NOT use this software    */
/* Some highlights of the use agreement are as follows:                 */
/* This software is intended for and must only be                       */
/* used for reference and in an evaluation laboratory environment.      */
/* It is provided on an as-is basis without charge and is subject to    */
/* alterations.                                                         */
/* It is the user's obligation to fully test the software in its        */
/* environment and to ensure proper functionality, qualification and    */
/* compliance with component specifications.                            */
/*                                                                      */
/* In the event the software deliverable includes the use of open       */
/* source components, the provisions of the governing open source       */
/* license agreement shall apply with respect to such software          */
/* deliverable.                                                         */
/*                                                                      */
/* FSLGROUP does not warrant that the deliverables do not infringe any  */
/* third party intellectual property right (IPR). In the event that     */
/* the deliverables infringe a third party IPR it is the sole           */
/* responsibility of the customer to obtain necessary licenses to       */
/* continue the usage of the deliverable.                               */
/*                                                                      */
/* FSLGROUP disclaims all warranties, whether express or implied,       */
/* in particular, but not limited to, warranties of merchantability     */
/* and fitness for a particular purpose.                                */
/*                                                                      */
/* To the maximum extent permitted by applicable law,                   */
/* FSLGROUP's liability is restricted to intentional misconduct         */
/* and gross negligence.                                                */
/*                                                                      */
/* FSLGROUP is not liable for consequential damages.                    */
/*                                                                      */
/* (V2.0)                                                               */
/************************************************************************/
/** \file main.c
 **
 ** This example demonstrates watch counter interrupt .
 ** IO polling wave can be observed at P61 during demonstration process.
 **
 ** History:
 **   - 2014-02-21  0.0.1  Edison Zhang        First version.
 **
 ******************************************************************************/

/******************************************************************************/
/* Include files                                                              */
/******************************************************************************/
#include "pdl_header.h"

/*---------------------------------------------------------------------------*/
/* local functions prototypes                                                */
/*---------------------------------------------------------------------------*/
static void IoInit(void);
static void IoPolling(void);
static void Delay(void);
static void WcDemoUseMainOsc(void);
static void WcDemoUseSubOsc(void);
static void WcDemoUseMainCr(void);
static void WcDemoUseSubCr(void);

/*---------------------------------------------------------------------------*/
/* local data                                                                */
/*---------------------------------------------------------------------------*/
static uint16_t m_u16ValidFlag = 0;

/*---------------------------------------------------------------------------*/
/* global functions                                                          */
/*---------------------------------------------------------------------------*/

/*!
 ******************************************************************************
 ** \brief Watch counter polling example code
 **
 ** 1. Demo Watch counter with Main Osc.
 ** 2. Demo Watch counter with Sub Osc.
 ** 3. Demo Watch counter with Main Cr.
 ** 4. Demo Watch counter with Sub Osc.
 ******************************************************************************
 */
int32_t main(void)
{

#ifdef DEBUG_PRINT
    printf("==================================================\n");
    printf("Watch Counter Polling Example Program Start \n");
    printf("==================================================\n");
#endif
    WcDemoUseMainOsc();
    WcDemoUseSubOsc();
    WcDemoUseMainCr();
    WcDemoUseSubCr();
#ifdef DEBUG_PRINT
    printf("==================================================\n");
    printf("Watch Counter Polling Example Program End \n");
    printf("==================================================\n");
#endif
    while(1);
}

/*!
 ******************************************************************************
 ** \brief  Initialize the IO (P38) to be tested
 **
 ** \param  none
 **
 ** \return none
 ******************************************************************************
 */
void IoInit(void)
{
    Gpio1pin_InitOut(GPIO1PIN_P61, Gpio1pin_InitDirectionOutput);
    return;
}

/*!
 ******************************************************************************
 ** \brief  Polling the IO (P38)
 **
 ** \param  none
 **
 ** \return none
 ******************************************************************************
 */
void IoPolling(void)
{
    /* ignore first trigger in order to synchronize with PCLK*/
    if(!m_u16ValidFlag)
    {
        m_u16ValidFlag++;
        return;
    }
    m_u16ValidFlag++;
    Gpio1pin_Put(GPIO1PIN_P61, (m_u16ValidFlag%2));
    return;
}

/*
 ******************************************************************************
 ** \brief Delay requested seconds approximately
 ******************************************************************************
 */
static void Delay(void)
{
    int32_t u8Cnt1,u8Cnt2;

    u8Cnt2 = SystemCoreClock;

    u8Cnt1 = 0;
    while (1)
    {
        u8Cnt1++;
        if (u8Cnt1 > u8Cnt2)
        {
            break;
        }
    }
}
/*!
 ******************************************************************************
 ** \brief Watch counter polling example code
 **
 ** 1. Initialization.
 ** 2. Set counter clock as MainOsc.
 ** 3. Enable count operation and delay some time.
 ** 4. Stop counter.
 ******************************************************************************
 */
static void WcDemoUseMainOsc(void)
{
    stc_wc_config_t stcWcConfig;
    stc_wc_pres_clk_t stcWcPresClk;
    uint32_t u32Cnt=0UL;

    // Initialize GPIO for watch counter demo
    IoInit();
#ifdef DEBUG_PRINT
    printf("1st demo\n");
    printf("Source clock = Main OSC \n");
    m_u16ValidFlag = 0;
#endif
    // Select watch counter prescaler source clock as Main OSC
    stcWcPresClk.enInputClk = WcPresInClkMainOsc;
    // Select watch counter source clock WCCK0
    stcWcConfig.enCntClk = WcCntClkWcck0;
    // Set watch counter reload value
    stcWcConfig.u8ReloadValue = 1;
    // Set counter value, 2^20/4000000 = 0.26S, 1 step = 0.26S
    stcWcPresClk.enOutputClk = WcPresOutClkArray6;

    Wc_Pres_SelClk(&stcWcPresClk);
    Wc_Pres_EnableDiv();

    Wc_Init(&stcWcConfig);
#ifdef DEBUG_PRINT
    printf("==================================================\n");
    printf("Start counter, waiting several seconds approximately\n");
#endif
    // Start counter
    Wc_EnableCount();
    // Delay some time
    do
    {
        if( PdlSet == Wc_GetIntFlag())
        {
            Wc_ClearIntFlag();
            IoPolling();
        }
        u32Cnt++;
    } while(u32Cnt <2500000UL);
#ifdef DEBUG_PRINT
    printf("==================================================\n");
    printf("Stop counter, waiting several seconds approximately\n");
    printf("\n");
#endif
    // Stop counter
    Wc_DisableCount();
    // Delay some time
    Delay();
}
/*!
 ******************************************************************************
 ** \brief Watch counter polling example code
 **
 ** 1. Initialization.
 ** 2. Set counter clock as SubOsc.
 ** 3. Enable count operation and delay some time.
 ** 4. Stop counter.
 ******************************************************************************
 */
static void WcDemoUseSubOsc(void)
{
    stc_wc_config_t stcWcConfig;
    stc_wc_pres_clk_t stcWcPresClk;
    uint32_t u32Cnt=0UL;

    // Initialize GPIO for watch counter demo
    IoInit();
#ifdef DEBUG_PRINT
    printf("2nd demo\n"); // TODO_Lv
    printf("Source clock = Sub OSC \n");
    m_u16ValidFlag = 0;
#endif
    // Select watch counter prescaler source clock as Sub OSC
    stcWcPresClk.enInputClk = WcPresInClkSubOsc;
    // Select watch counter source clock WCCK3
    stcWcConfig.enCntClk = WcCntClkWcck3;
    // Set watch counter reload value
    stcWcConfig.u8ReloadValue = 1;
    // Set counter value, 2^15/32768, 1 step = 1S
    stcWcPresClk.enOutputClk = WcPresOutClkArray0;

    Wc_Pres_SelClk(&stcWcPresClk);
    Wc_Pres_EnableDiv();

    Wc_Init(&stcWcConfig);
#ifdef DEBUG_PRINT
    printf("==================================================\n");
    printf("Start counter, waiting several seconds approximately\n");
#endif
    // Start counter
    Wc_EnableCount();
    // Delay some time
    do
    {
        if( PdlSet == Wc_GetIntFlag())
        {
            Wc_ClearIntFlag();
            IoPolling();
        }
        u32Cnt++;
    } while(u32Cnt <6500000UL);
#ifdef DEBUG_PRINT
    printf("==================================================\n");
    printf("Stop counter, waiting several seconds approximately\n");
    printf("\n");
#endif
    // Stop counter
    Wc_DisableCount();
    // Delay some time
    Delay();
}

/*!
 ******************************************************************************
 ** \brief Watch counter polling example code
 **
 ** 1. Initialization.
 ** 2. Set counter clock as MainCr.
 ** 3. Enable count operation and delay some time.
 ** 4. Stop counter.
 ******************************************************************************
 */
static void WcDemoUseMainCr(void)
{
    stc_wc_config_t stcWcConfig;
    stc_wc_pres_clk_t stcWcPresClk;
    uint32_t u32Cnt=0UL;

    // Initialize GPIO for watch counter demo
    IoInit();
#ifdef DEBUG_PRINT
    printf("3rd demo\n");
    printf("Source clock = High CR \n");
    m_u16ValidFlag = 0;
#endif
    // Select watch counter prescaler source clock as High CR
    stcWcPresClk.enInputClk = WcPresInClkHighCr;
    // Select watch counter source clock WCCK0
    stcWcConfig.enCntClk = WcCntClkWcck0;
    stcWcConfig.u8ReloadValue = 1;
    // Set counter value, 2^20/4000000 = 0.26S, 1 step = 0.26S
    stcWcPresClk.enOutputClk = WcPresOutClkArray6;

    Wc_Pres_SelClk(&stcWcPresClk);
    Wc_Pres_EnableDiv();

    Wc_Init(&stcWcConfig);
#ifdef DEBUG_PRINT
    printf("==================================================\n");
    printf("Start counter, waiting several seconds approximately\n");
#endif
    // Start counter
    Wc_EnableCount();
    // Delay some time
    do
    {
        if( PdlSet == Wc_GetIntFlag())
        {
            Wc_ClearIntFlag();
            IoPolling();
        }
        u32Cnt++;
    } while(u32Cnt <2500000UL);
#ifdef DEBUG_PRINT
    printf("==================================================\n");
    printf("Stop counter, waiting several seconds approximately\n");
    printf("\n");
#endif
    // Stop counter
    Wc_DisableCount();
    // Delay some time
    Delay();
}

/*!
 ******************************************************************************
 ** \brief Watch counter polling example code
 **
 ** 1. Initialization.
 ** 2. Set counter clock as SubCr.
 ** 3. Enable count operation and delay some time.
 ** 4. Stop counter.
 ******************************************************************************
 */
static void WcDemoUseSubCr(void)
{
    stc_wc_config_t stcWcConfig;
    stc_wc_pres_clk_t stcWcPresClk;
    uint32_t u32Cnt=0;

    // Initialize GPIO for watch counter demo
    IoInit();
#ifdef DEBUG_PRINT
    printf("4th demo\n");
    printf("Source clock = Low CR \n");
    m_u16ValidFlag = 0;
#endif
    // Select watch counter prescaler source clock as Low CR
    stcWcPresClk.enInputClk = WcPresInClkLowCr;
    // Select watch counter source clock WCCK0
    stcWcConfig.enCntClk = WcCntClkWcck1;
    // Set watch counter reload value
    stcWcConfig.u8ReloadValue = 1;
    // Set counter value, 2^10/100000, 1 step = 10.24mS
    stcWcPresClk.enOutputClk = WcPresOutClkArray4;

    Wc_Pres_SelClk(&stcWcPresClk);
    Wc_Pres_EnableDiv();

    Wc_Init(&stcWcConfig);
#ifdef DEBUG_PRINT
    printf("==================================================\n");
    printf("Start counter, waiting several seconds approximately\n");
#endif
    // Start counter
    Wc_EnableCount();
    // Delay some time
    do
    {
        if( PdlSet == Wc_GetIntFlag())
        {
            Wc_ClearIntFlag();
            IoPolling();
        }
        u32Cnt++;
    } while(u32Cnt <2500000UL);
#ifdef DEBUG_PRINT
    printf("==================================================\n");
    printf("Stop counter, waiting several seconds approximately\n");
    printf("\n");
#endif
    // Stop counter
    Wc_DisableCount();
    // Delay some time
    Delay();
}
/*****************************************************************************/
/* END OF FILE */
