/*******************************************************************************
* Copyright (C) 2013 Spansion LLC. All Rights Reserved. 
*
* This software is owned and published by: 
* Spansion LLC, 915 DeGuigne Dr. Sunnyvale, CA  94088-3453 ("Spansion").
*
* BY DOWNLOADING, INSTALLING OR USING THIS SOFTWARE, YOU AGREE TO BE BOUND 
* BY ALL THE TERMS AND CONDITIONS OF THIS AGREEMENT.
*
* This software contains source code for use with Spansion 
* components. This software is licensed by Spansion to be adapted only 
* for use in systems utilizing Spansion components. Spansion shall not be 
* responsible for misuse or illegal use of this software for devices not 
* supported herein.  Spansion is providing this software "AS IS" and will 
* not be responsible for issues arising from incorrect user implementation 
* of the software.  
*
* SPANSION MAKES NO WARRANTY, EXPRESS OR IMPLIED, ARISING BY LAW OR OTHERWISE,
* REGARDING THE SOFTWARE (INCLUDING ANY ACOOMPANYING WRITTEN MATERIALS), 
* ITS PERFORMANCE OR SUITABILITY FOR YOUR INTENDED USE, INCLUDING, 
* WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY, THE IMPLIED 
* WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE OR USE, AND THE IMPLIED 
* WARRANTY OF NONINFRINGEMENT.  
* SPANSION SHALL HAVE NO LIABILITY (WHETHER IN CONTRACT, WARRANTY, TORT, 
* NEGLIGENCE OR OTHERWISE) FOR ANY DAMAGES WHATSOEVER (INCLUDING, WITHOUT 
* LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS, BUSINESS INTERRUPTION, 
* LOSS OF BUSINESS INFORMATION, OR OTHER PECUNIARY LOSS) ARISING FROM USE OR 
* INABILITY TO USE THE SOFTWARE, INCLUDING, WITHOUT LIMITATION, ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL OR CONSEQUENTIAL DAMAGES OR LOSS OF DATA, 
* SAVINGS OR PROFITS, 
* EVEN IF SPANSION HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. 
* YOU ASSUME ALL RESPONSIBILITIES FOR SELECTION OF THE SOFTWARE TO ACHIEVE YOUR
* INTENDED RESULTS, AND FOR THE INSTALLATION OF, USE OF, AND RESULTS OBTAINED 
* FROM, THE SOFTWARE.  
*
* This software may be replicated in part or whole for the licensed use, 
* with the restriction that this Disclaimer and Copyright notice must be 
* included with each copy of this software, whether used in part or whole, 
* at all times.  
*/
/******************************************************************************/
/** \file dma.h
 **
 ** Headerfile for DMA functions
 **  
 ** History:
 **   - 2014-02-19  0.1  Edison Zhang  First version
 **   - 2014-03-04  0.2  Edison Zhang  Delete USB triggers in the DMA hardware
 **                                    trigger list
 **
 ******************************************************************************/

#ifndef __DMA_H__
#define __DMA_H__

/******************************************************************************/
/* Include files                                                              */
/******************************************************************************/
#include "mcu.h"
#include "pdl_user.h"

#if (defined(PDL_PERIPHERAL_DMA_ACTIVE))

/* C binding of definitions if building with C++ compiler */
#ifdef __cplusplus
extern "C"
{
#endif

/**
 ******************************************************************************
 ** \defgroup DmaGroup Direct Memory Access (DMA)
 **
 ** Provided functions of DMA module:
 ** 
 ** - Dma_Init_Channel()
 ** - Dma_DeInit_Channel() 
 ** - Dma_SetChannel() 
 ** - Dma_GetStopCause()
 ** - Dma_Enable()
 ** - Dma_Disable()
 **
 **  Dma_Init_Channel() initializes a DMA channel with the structure type
 **  #stc_dma_config_t. 
 ** 
 **  Dma_Init_Channel() resets a DMA channel to its initial status.
 **
 **  Dma_SetChannel() can enable/disable and triggers a channel by software. 
 **
 **  Dma_GetStopCause() gets the DMA completion cause.
 **
 **  Dma_Enable() enables the entire DMA module and Dma_Disable() disables 
 **  entire DMA module.
 ** 
 ******************************************************************************/
//@{
  
/******************************************************************************/
/* Global pre-processor symbols/macros ('#define')                            */
/******************************************************************************/

#define DMA_CH_COUNT        (4u)
#define DMA_MAX_CH_INDEX    (3u) 
                             
// register address offset
#define DMA_DMACA_OFFSET 0x00ul
#define DMA_DMACB_OFFSET 0x04ul
#define DMA_DMASA_OFFSET 0x08ul
#define DMA_DMADA_OFFSET 0x0Cul
    
/******************************************************************************/
/* Local type definitions ('typedef')                                         */
/******************************************************************************/

/**
 ******************************************************************************
 ** \brief DMA transfer data width
 ******************************************************************************/
typedef enum en_dma_transfermode
{
  DmaBlockTransfer  = 0,  ///< Block Transfer
  DmaBurstTransfer  = 1,  ///< Burst transfer
  DmaDemandTransfer = 2   ///< Demand transfer
} en_dma_transfermode_t ;

/**
 ******************************************************************************
 ** \brief DMA transfer data width
 ******************************************************************************/
typedef enum en_dma_transferwidth
{
  Dma8Bit  = 0,  ///< 8 bit transfer via DMA
  Dma16Bit = 1,  ///< 16 bit transfer via DMA
  Dma32Bit = 2   ///< 32 bit transfer via DMA
} en_dma_transferwidth_t ;

/**
 ******************************************************************************
 ** \brief DMA IDREQ number
 ******************************************************************************/
typedef enum en_dma_idreq
{
  Adc0        = 5,   ///< Scan conversion interrupt signal from A/D converter unit0
  Adc1        = 6,   ///< Scan conversion interrupt signal from A/D converter unit1
  Adc2        = 7,   ///< Scan conversion interrupt signal from A/D converter unit2
  Bt0Irq0     = 8,   ///< Interrupt signal from IRQ0 of base timer ch.0
  Bt2Irq0     = 9,   ///< Interrupt signal from IRQ0 of base timer ch.2
  Bt4Irq0     = 10,  ///< Interrupt signal from IRQ0 of base timer ch.4
  Bt6Irq0     = 11,  ///< Interrupt signal from IRQ0 of base timer ch.6
  MfsRx0      = 12,  ///< Receiving interrupt signal from MFS ch.0
  MfsTx0      = 13,  ///< Sending interrupt signal from MFS ch.0   
  MfsRx1      = 14,  ///< Receiving interrupt signal from MFS ch.1
  MfsTx1      = 15,  ///< Sending interrupt signal from MFS ch.1
  MfsRx2      = 16,  ///< Receiving interrupt signal from MFS ch.2
  MfsTx2      = 17,  ///< Sending interrupt signal from MFS ch.2
  MfsRx3      = 18,  ///< Receiving interrupt signal from MFS ch.3
  MfsTx3      = 19,  ///< Sending interrupt signal from MFS ch.3 
  MfsRx4      = 20,  ///< Receiving interrupt signal from MFS ch.4
  MfsTx4      = 21,  ///< Sending interrupt signal from MFS ch.4 
  MfsRx5      = 22,  ///< Receiving interrupt signal from MFS ch.5
  MfsTx5      = 23,  ///< Sending interrupt signal from MFS ch.5 
  MfsRx6      = 24,  ///< Receiving interrupt signal from MFS ch.6
  MfsTx6      = 25,  ///< Sending interrupt signal from MFS ch.6 
  MfsRx7      = 26,  ///< Receiving interrupt signal from MFS ch.7
  MfsTx7      = 27,  ///< Sending interrupt signal from MFS ch.7 
  ExtIrq0     = 28,  ///< Interrupt signal from external interrupt unit ch.0
  ExtIrq1     = 29,  ///< Interrupt signal from external interrupt unit ch.1    
  ExtIrq2     = 30,  ///< Interrupt signal from external interrupt unit ch.2
  ExtIrq3     = 31,  ///< Interrupt signal from external interrupt unit ch.3 
  Software    = 1234 ///< Software Demand (just a high number)    
} en_dma_idreq_t ;

/**
 ******************************************************************************
 ** \brief DMA stop cause
 ******************************************************************************/
typedef enum dma_stop_cause
{
  DmaStopByAddrOverflow        = 0,  ///< DMA stops by address overflow
  DmaStopByTransferStopRqst    = 1,  ///< DMA stops by transfer stop request
  DmaStopBySrcAccessError      = 2,  ///< DMA stops by source access error
  DmaStopByDstAccessError      = 3,  ///< DMA stops by destination access error
  DmaStopByTransferComplete    = 4,  ///< DMA stops by transfer completion
  DmaStopByTransferPause       = 5,  ///< DMA stops by transfer pause
    
}en_dma_stop_cause_t;

/**
 ******************************************************************************
 ** \brief DMA interrupt selection
 ******************************************************************************/
typedef struct stc_dma_int_sel
{
    boolean_t bCompleteInt;   ///< Select DMA transfer completion interrupt
    boolean_t bErrorInt;      ///< Select DMA transfer error interrupt
    
}stc_dma_int_sel_t;

/**
 ******************************************************************************
 ** \brief DMA interrupt callback function
 ******************************************************************************/
typedef struct stc_dma_int_cb
{
    func_ptr_t pfnCompleteCallback;   ///< Pointer to DMA transfer completion interrupt callback function
    func_ptr_arg1_t pfnErrorCallback; ///< Pointer to DMA transfer error interrupt callback function
    
}stc_dma_int_cb_t;

/**
 ******************************************************************************
 ** \brief Datatype for holding internal data needed for DMA
 ******************************************************************************/
typedef struct stc_dma_intern_data
{
  func_ptr_t      apfnDmaCallback[DMA_CH_COUNT];       ///< Callback pointer array
  func_ptr_arg1_t apfnDmaErrorCallback[DMA_CH_COUNT];  ///< Error Callback pointer array

} stc_dma_intern_data_t ;

/**
 ******************************************************************************
 ** \brief DMA configuration
 ******************************************************************************/
typedef struct stc_dma_config
{
  en_dma_idreq_t         enDmaIdrq ;             ///< ID Request number (see #en_dma_idreq_t for details)
  uint8_t                u8BlockCount ;          ///< Block counter
  uint16_t               u16TransferCount ;      ///< Transfer counter
  en_dma_transfermode_t  enTransferMode ;        ///< DMA transfer mode (see #en_dma_transfermode_t for details)
  en_dma_transferwidth_t enTransferWdith ;       ///< DMA transfer width (see #en_dma_transferwidth_t for details)
  uint32_t               u32SourceAddress;       ///< Source address
  uint32_t               u32DestinationAddress ; ///< Destination address
  boolean_t              bFixedSource ;          ///< TRUE = source address not increased
  boolean_t              bFixedDestination ;     ///< TRUE = destination address not increased
  boolean_t              bReloadCount ;          ///< TRUE = count is reloaded
  boolean_t              bReloadSource ;         ///< TRUE = source address is reloaded
  boolean_t              bReloadDestination ;    ///< TRUE = destination address is reloaded
  boolean_t              bEnableBitMask ;        ///< FALSE = Clear EB (bEnable) bit on completion (mandatory for transfer end!)
} stc_dma_config_t ;

/******************************************************************************/
/* Global variable definitions ('extern')                                     */
/******************************************************************************/
extern stc_dma_intern_data_t stcDmaInternData;

/******************************************************************************/
/* Global function prototypes (definition in C source)                        */
/******************************************************************************/  
#if (PDL_INTERRUPT_ENABLE_DMA0 == PDL_ON) || (PDL_INTERRUPT_ENABLE_DMA1 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_DMA2 == PDL_ON) || (PDL_INTERRUPT_ENABLE_DMA3 == PDL_ON) 
void DmaIrqHandler(uint8_t u8DmaChannel) ;

en_result_t Dma_EnableInt(uint8_t u8DmaChannel, 
                          stc_dma_int_sel_t* pstcIntSel,
                          stc_dma_int_cb_t* pstcIntCb  );
en_result_t Dma_DisableInt(uint8_t u8DmaChannel, 
                          stc_dma_int_sel_t* pstcIntSel );
#endif

en_result_t Dma_Init_Channel(uint8_t u8DmaChannel, volatile stc_dma_config_t* pstcConfig);

en_result_t Dma_DeInit_Channel(uint8_t u8DmaChannel);

en_result_t Dma_SetChannel (uint8_t u8DmaChannel, 
                            boolean_t bEnable,
                            boolean_t bPause,
                            boolean_t bSoftwareTrigger) ;

en_dma_stop_cause_t Dma_GetStopCause(uint8_t u8DmaChannel);

en_result_t Dma_Enable(void) ;

en_result_t Dma_Disable(void) ;



//@} // DmaGroup

#ifdef __cplusplus
}
#endif

#endif // #if (defined(PDL_PERIPHERAL_DMA_ACTIVE))

#endif /* __DMA_H__ */
/******************************************************************************/
/* EOF (not truncated)                                                        */
/******************************************************************************/
