/*******************************************************************************
* Copyright (C) 2013 Spansion LLC. All Rights Reserved. 
*
* This software is owned and published by: 
* Spansion LLC, 915 DeGuigne Dr. Sunnyvale, CA  94088-3453 ("Spansion").
*
* BY DOWNLOADING, INSTALLING OR USING THIS SOFTWARE, YOU AGREE TO BE BOUND 
* BY ALL THE TERMS AND CONDITIONS OF THIS AGREEMENT.
*
* This software contains source code for use with Spansion 
* components. This software is licensed by Spansion to be adapted only 
* for use in systems utilizing Spansion components. Spansion shall not be 
* responsible for misuse or illegal use of this software for devices not 
* supported herein.  Spansion is providing this software "AS IS" and will 
* not be responsible for issues arising from incorrect user implementation 
* of the software.  
*
* SPANSION MAKES NO WARRANTY, EXPRESS OR IMPLIED, ARISING BY LAW OR OTHERWISE,
* REGARDING THE SOFTWARE (INCLUDING ANY ACOOMPANYING WRITTEN MATERIALS), 
* ITS PERFORMANCE OR SUITABILITY FOR YOUR INTENDED USE, INCLUDING, 
* WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY, THE IMPLIED 
* WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE OR USE, AND THE IMPLIED 
* WARRANTY OF NONINFRINGEMENT.  
* SPANSION SHALL HAVE NO LIABILITY (WHETHER IN CONTRACT, WARRANTY, TORT, 
* NEGLIGENCE OR OTHERWISE) FOR ANY DAMAGES WHATSOEVER (INCLUDING, WITHOUT 
* LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS, BUSINESS INTERRUPTION, 
* LOSS OF BUSINESS INFORMATION, OR OTHER PECUNIARY LOSS) ARISING FROM USE OR 
* INABILITY TO USE THE SOFTWARE, INCLUDING, WITHOUT LIMITATION, ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL OR CONSEQUENTIAL DAMAGES OR LOSS OF DATA, 
* SAVINGS OR PROFITS, 
* EVEN IF SPANSION HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. 
* YOU ASSUME ALL RESPONSIBILITIES FOR SELECTION OF THE SOFTWARE TO ACHIEVE YOUR
* INTENDED RESULTS, AND FOR THE INSTALLATION OF, USE OF, AND RESULTS OBTAINED 
* FROM, THE SOFTWARE.  
*
* This software may be replicated in part or whole for the licensed use, 
* with the restriction that this Disclaimer and Copyright notice must be 
* included with each copy of this software, whether used in part or whole, 
* at all times.  
*/
/******************************************************************************/
/** \file lpm.h
 **
 ** Headerfile for LPM functions
 **  
 ** History:
 **   - 2013-10-23  1.0  MWi  First version.
 **
 ******************************************************************************/

#ifndef __LPM_H__
#define __LPM_H__

/******************************************************************************/
/* Include files                                                              */
/******************************************************************************/
#include "mcu.h"
#include "pdl_user.h"

#if (defined(PDL_PERIPHERAL_LPM_ACTIVE))

/* C binding of definitions if building with C++ compiler */
#ifdef __cplusplus
extern "C"
{
#endif
  
/**
 ******************************************************************************
 ** \defgroup LpmGroup Low Power Modes Functions (LPM)
 **
 ** \note The deep standby mode may not be available for some of FM0+ product.
 **       Please refer to the hardware manual for the function availability.  
 ** 
 ** Provided functions of LPM module:
 **
 ** - Lpm_GoToStandByMode()
 ** - Lpm_ConfigDeepStbRetCause()
 ** - Lpm_ReadDeepStbRetCause()
 ** - Lpm_SetWkupPinLevel()
 ** - Lpm_ConfigSubClk()
 ** - Lpm_ConfigDeepStbRAMRetention()
 ** - Lpm_WriteBackupReg()
 ** - Lpm_ReadBackupReg() 
 **
 **  Lpm_GoToStandByMode() enters the low power consumption selected by 
 **  the parameter Lpm_GoToStandByMode#enMode. Since MCU enters into low
 **  power consumption mode after calling this funciton, this function will
 **  not return until MCU is waken up. There are also different between waken up
 **  from Standby mode and deep standby mode. When MCU is waken up from standby
 **  mode, the program of MCU will run after this function, but when MCU is 
 **  waken up from deep standby mode, the program of MCU will run from the reset
 **  vector.
 **
 **  Lpm_ConfigDeepStbRetCause() configures the return causes of deep standby
 **  mode and Lpm_ReadDeepStbRetCause() reads the return causes after MCU is 
 **  waken up. Note that Wakeup from WKUP0 pin is always enabled.
 ** 
 **  Lpm_SetWkupPinLevel() is used to set the WKUP pin active level.  
 ** 
 **  Lpm_ConfigSubClk() configures whether to supply Sub clock to RTC and CEC 
 **  in deep standby mode. Not all FM0P+ product has this function. Please 
 **  confirm it in the hardware manual.
 **
 **  Lpm_ConfigDeepStbRAMRetention() configures whether to retain RAM content
 **  after entering deep standby mode. Not all FM0P+ product has this function. 
 **  Plesae comfirm it in the hardware manual.
 **
 **  Lpm_WriteBackupReg() writes the backup registers, which can retain in the 
 **  deep standby mode. and Lpm_ReadBackupReg() reads the backup regisers after
 **  waken up from deep standby mode. Up to 16 byte backup registers can be used.
 **
 ******************************************************************************/
//@{

/******************************************************************************/
/* Global pre-processor symbols/macros ('#define')      
 ******************************************************************************/
#define WIFSR_WRTCI                 (1ul<<0)
#define WIFSR_WLVDI                 (1ul<<1)
#define WIFSR_WUI0                  (1ul<<2)
#define WIFSR_WUI1                  (1ul<<3)
#define WIFSR_WUI2                  (1ul<<4)
#define WIFSR_WUI3                  (1ul<<5)
#define WIFSR_WUI4                  (1ul<<6)
#define WIFSR_WUI5                  (1ul<<7)
#define WIFSR_WCEC0I                (1ul<<8)
#define WIFSR_WCEC1I                (1ul<<9)

#define WRFSR_WINITX                 (1ul<<0)
#define WRFSR_WLVDH                  (1ul<<1)  
     
/*!
 ******************************************************************************
 ** \brief define standby mode type
 ******************************************************************************
 */
typedef enum en_lpm_mode
{
    StbSleepMode    = 0u,    //!< standby sleep mode
    StbTimerMode    = 1u,    //!< standby timer mode    
    StbStopMode     = 2u,    //!< standby stop mode     
    StbRtcMode      = 3u,    //!< standby RTC mode 
    DeepStbRtcMode  = 4u,    //!< deep standby RTC mode
    DeepStbStopMode = 5u,    //!< deep standby stop mode      
    
} en_lpm_mode_t;
 
/*!
 ******************************************************************************
 ** \brief Deep standby mode return cause
 ******************************************************************************
 */
typedef struct stc_ret_cause
{
    boolean_t   bRtcEn;     //!< Return from RTC interrupt enable
    boolean_t   bLvdEn;     //!< Return from LVD interrupt enable
    boolean_t   bWakeup1En; //!< Return from Wakeup1 pin enable
    boolean_t   bWakeup2En; //!< Return from Wakeup2 pin enable
    boolean_t   bWakeup3En; //!< Return from Wakeup3 pin enable
    boolean_t   bWakeup4En; //!< Return from Wakeup4 pin enable
    boolean_t   bWakeup5En; //!< Return from Wakeup5 pin enable
    boolean_t   bCec0En;    //!< Return from CEC0 interrupt enable
    boolean_t   bCec1En;    //!< Return from CEC1 interrupt enable
    
}stc_dstb_ret_cause_t;


/*!
 ******************************************************************************
 ** \brief define deep standby mode return cause flag
 ******************************************************************************
 */
typedef enum en_ret_cause_flag
{
    DeepStbNoFlag    = 0u,     //!< No flag set
    DeepStbInitx     = 1u,     //!< return from INITX input reset
    DeepStbLvdReset  = 2u,     //!< return from LVD reset
    DeepStbRtcInt    = 3u,     //!< return from RTC interrupt
    DeepStbLvdInt    = 4u,     //!< return from LVD interrupt
    DeepStbWkupPin0  = 5u,     //!< return from Wkup pin1 detection
    DeepStbWkupPin1  = 6u,     //!< return from Wkup pin1 detection
    DeepStbWkupPin2  = 7u,     //!< return from Wkup pin2 detection
    DeepStbWkupPin3  = 8u,     //!< return from Wkup pin3 detection
    DeepStbWkupPin4  = 9u,     //!< return from Wkup pin4 detection
    DeepStbWkupPin5  = 10u,    //!< return from Wkup pin5 detection
    DeepStbCec0      = 11u,    //!< return from Wkup CEC1 reception interrupt
    DeepStbCec1      = 12u,    //!< return from Wkup CEC2 reception interrupt
} en_dstb_ret_cause_t;

/*!
******************************************************************************
 ** \brief define wkup pin index
******************************************************************************
*/
typedef enum en_dstb_wkup_pin
{
    WkupPin1 = 0u,      //!< index of wkup pin 1
    WkupPin2 = 1u,      //!< index of wkup pin 2
    WkupPin3 = 2u,      //!< index of wkup pin 3
    WkupPin4 = 3u,      //!< index of wkup pin 4
    WkupPin5 = 4u,      //!< index of wkup pin 5
    
} en_dstb_wkup_pin_t;

/*!
 ******************************************************************************
 ** \brief Valid level of wakeup pin
 ******************************************************************************
 */
typedef enum en_wkup_valid_level
{
    WkupLowLevelValid   = 0u,  //!< Set low level as valid level
    WkupHighLevelValid  = 1u,  //!< Set high level as valid level
    
}en_wkup_valid_level_t;

/*!
 ******************************************************************************
 ** \brief backup register index
 ******************************************************************************
 */
typedef enum en_dstb_bakup_reg
{
    BackupReg1 = 1u,      //!< index of backup register 1
    BackupReg2 = 2u,      //!< index of backup register 2
    BackupReg3 = 3u,      //!< index of backup register 3
    BackupReg4 = 4u,      //!< index of backup register 4
    BackupReg5 = 5u,      //!< index of backup register 5
    BackupReg6 = 6u,      //!< index of backup register 6
    BackupReg7 = 7u,      //!< index of backup register 7
    BackupReg8 = 8u,      //!< index of backup register 8
    BackupReg9 = 9u,      //!< index of backup register 9
    BackupReg10 = 10u,    //!< index of backup register 10
    BackupReg11 = 11u,    //!< index of backup register 11
    BackupReg12 = 12u,    //!< index of backup register 12
    BackupReg13 = 13u,    //!< index of backup register 13
    BackupReg14 = 14u,    //!< index of backup register 14
    BackupReg15 = 15u,    //!< index of backup register 15
    BackupReg16 = 16u,    //!< index of backup register 16
    
} en_dstb_bakup_reg_t;

/******************************************************************************/
/* Global variable definitions ('extern')                                     */
/******************************************************************************/

/******************************************************************************/
/* Global function prototypes (definition in C source)                        */
/******************************************************************************/
void Lpm_GoToStandByMode(en_lpm_mode_t enMode, boolean_t bIoRemain);
en_result_t Lpm_ConfigDeepStbRetCause(stc_dstb_ret_cause_t* pstcCause);
en_dstb_ret_cause_t Lpm_ReadDeepStbRetCause(void);
en_result_t Lpm_SetWkupPinLevel(en_dstb_wkup_pin_t enPinIndex, en_wkup_valid_level_t enLevel);
en_result_t Lpm_ConfigSubClk(boolean_t bSupplyCec, boolean_t bSupplyRtc);
en_result_t Lpm_ConfigDeepStbRAMRetention(boolean_t bRamRetain);
en_result_t Lpm_WriteBackupReg(en_dstb_bakup_reg_t enRegIndex, uint8_t u8Data);
uint8_t Lpm_ReadBackupReg(en_dstb_bakup_reg_t enRegIndex);

//@} // LpmGroup

#ifdef __cplusplus
}
#endif

#endif // #if (defined(PDL_PERIPHERAL_LPM_ACTIVE))

#endif /* __LPM_H__ */
/******************************************************************************/
/* EOF (not truncated)                                                        */
/******************************************************************************/
