/*******************************************************************************
* Copyright (C) 2013 Spansion LLC. All Rights Reserved. 
*
* This software is owned and published by: 
* Spansion LLC, 915 DeGuigne Dr. Sunnyvale, CA  94088-3453 ("Spansion").
*
* BY DOWNLOADING, INSTALLING OR USING THIS SOFTWARE, YOU AGREE TO BE BOUND 
* BY ALL THE TERMS AND CONDITIONS OF THIS AGREEMENT.
*
* This software contains source code for use with Spansion 
* components. This software is licensed by Spansion to be adapted only 
* for use in systems utilizing Spansion components. Spansion shall not be 
* responsible for misuse or illegal use of this software for devices not 
* supported herein.  Spansion is providing this software "AS IS" and will 
* not be responsible for issues arising from incorrect user implementation 
* of the software.  
*
* SPANSION MAKES NO WARRANTY, EXPRESS OR IMPLIED, ARISING BY LAW OR OTHERWISE,
* REGARDING THE SOFTWARE (INCLUDING ANY ACOOMPANYING WRITTEN MATERIALS), 
* ITS PERFORMANCE OR SUITABILITY FOR YOUR INTENDED USE, INCLUDING, 
* WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY, THE IMPLIED 
* WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE OR USE, AND THE IMPLIED 
* WARRANTY OF NONINFRINGEMENT.  
* SPANSION SHALL HAVE NO LIABILITY (WHETHER IN CONTRACT, WARRANTY, TORT, 
* NEGLIGENCE OR OTHERWISE) FOR ANY DAMAGES WHATSOEVER (INCLUDING, WITHOUT 
* LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS, BUSINESS INTERRUPTION, 
* LOSS OF BUSINESS INFORMATION, OR OTHER PECUNIARY LOSS) ARISING FROM USE OR 
* INABILITY TO USE THE SOFTWARE, INCLUDING, WITHOUT LIMITATION, ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL OR CONSEQUENTIAL DAMAGES OR LOSS OF DATA, 
* SAVINGS OR PROFITS, 
* EVEN IF SPANSION HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. 
* YOU ASSUME ALL RESPONSIBILITIES FOR SELECTION OF THE SOFTWARE TO ACHIEVE YOUR
* INTENDED RESULTS, AND FOR THE INSTALLATION OF, USE OF, AND RESULTS OBTAINED 
* FROM, THE SOFTWARE.  
*
* This software may be replicated in part or whole for the licensed use, 
* with the restriction that this Disclaimer and Copyright notice must be 
* included with each copy of this software, whether used in part or whole, 
* at all times.  
*/
/******************************************************************************/
/** \file rtc.c
 **
 ** A detailed description is available at 
 ** @link RtcGroup RTC Module description @endlink
 **
 ** History:
 **   - 2014-02-04  1.0  Edison Zhang   First version.
 **   - 2014-03-10  1.1  Edison Zhang   Update compare value of 
 **                                     frequency correction cycle in Rtc_Init()
 **
 ******************************************************************************/

/******************************************************************************/
/* Include files                                                              */
/******************************************************************************/
#include "rtc.h"

#if (defined(PDL_PERIPHERAL_RTC_ACTIVE))

/**
 ******************************************************************************
 ** \addtogroup RtcGroup
 ******************************************************************************/
//@{

/******************************************************************************/
/* Local pre-processor symbols/macros ('#define')                             */
/******************************************************************************/
/* Timeout count */
#define RTC_TIMEOUT              (SystemCoreClock/10)
/* Error code for mktime */
#define RTC_ERR                 (-1)
/* convert DEC to BCD */
#define DecToBcd(x)           ((((x)/10)<<4) + ((x)%10))
/* convert BCD to DEC */
#define BcdToDec(x)           ((((x)>>4)*10) + ((x)&0x0F))

/******************************************************************************/
/* Global variable definitions (declared in header file with 'extern')        */
/******************************************************************************/

/******************************************************************************/
/* Local type definitions ('typedef')                                         */
/******************************************************************************/

/******************************************************************************/
/* Local function prototypes ('static')                                       */
/******************************************************************************/
static void RtcToTm( stc_rtc_time_t* pstcRtcTime,
                            struct tm*      pstcTime);
static void RtcDisableNvic(void);
static void RtcRestoreNvic(void);

#if (PDL_INTERRUPT_ENABLE_RTC == PDL_ON)
static void RtcInitIrq(void);
static void RtcDeInitIrq(void);
#endif

/******************************************************************************/
/* Local variable definitions ('static')                                      */
/******************************************************************************/
#if (PDL_INTERRUPT_ENABLE_RTC == PDL_ON)
static stc_rtc_intern_data_t stcRtcInternData;
#endif
static uint32_t u32NvicData;

/******************************************************************************/
/* Function implementation - global ('extern') and local ('static')           */
/******************************************************************************/

/**
 ******************************************************************************
 ** \brief Disable RTC NVIC and save original value
 ******************************************************************************/
static void RtcDisableNvic(void)
{
    u32NvicData = NVIC->ISER[(uint32_t)((int32_t)OSC_PLL_WC_RTC_IRQn) >> 5];
    NVIC->ICER[((uint32_t)(OSC_PLL_WC_RTC_IRQn) >> 5)] = (1 << ((uint32_t)(OSC_PLL_WC_RTC_IRQn) & 0x1F));
}

/**
 ******************************************************************************
 ** \brief Restore RTC NVIC
 ******************************************************************************/
static void RtcRestoreNvic(void)
{
    NVIC->ISER[(uint32_t)((int32_t)OSC_PLL_WC_RTC_IRQn) >> 5] = u32NvicData;
}


/**
 ******************************************************************************
 ** \brief Convert RTC time structure to time.h tm structure
 **
 ** \param [in]  pstcRtcTime    RTC Time structure
 ** \param [out] pstcTime       tm Time structure
 **
 ******************************************************************************/
static void RtcToTm( stc_rtc_time_t* pstcRtcTime,
                            struct tm*      pstcTime)
{
    pstcTime->tm_year  = (int)(pstcRtcTime->u16Year - 1900);
    pstcTime->tm_mon   = (int)(pstcRtcTime->u8Month - 1);
    pstcTime->tm_mday  = (int)(pstcRtcTime->u8Day);
    pstcTime->tm_sec   = (int)(pstcRtcTime->u8Second);
    pstcTime->tm_min   = (int)(pstcRtcTime->u8Minute);
    pstcTime->tm_hour  = (int)(pstcRtcTime->u8Hour);
    pstcTime->tm_isdst = 0;
} /* RtcToTm */

#if (PDL_INTERRUPT_ENABLE_RTC == PDL_ON)
/**
 ******************************************************************************
 ** \brief Device dependent initialization of interrupts according CMSIS with
 **        level defined in pdl_user.h
 **
 ******************************************************************************/
static void RtcInitIrq(void)
{
    NVIC_ClearPendingIRQ(OSC_PLL_WC_RTC_IRQn);
    NVIC_EnableIRQ(OSC_PLL_WC_RTC_IRQn);
    NVIC_SetPriority(OSC_PLL_WC_RTC_IRQn, PDL_IRQ_LEVEL_CLK_WC_RTC);

} /* RtcInitIrq */

/**
 ******************************************************************************
 ** \brief Device dependent de-initialization of interrupts according CMSIS with
 **        level defined in pdl.h
 **
 ******************************************************************************/
static void RtcDeInitIrq(void)
{
    NVIC_ClearPendingIRQ(OSC_PLL_WC_RTC_IRQn);
    NVIC_DisableIRQ(OSC_PLL_WC_RTC_IRQn);
    NVIC_SetPriority(OSC_PLL_WC_RTC_IRQn, PDL_DEFAULT_INTERRUPT_LEVEL);
} /* RtcDeInitIrq */

/**
 ******************************************************************************
 ** \brief RTC  interrupt service routine.
 ******************************************************************************/
void Rtc_IrqHandler(void)
{
    stc_rtc_wtcr1_field_t stcWtcr1;
    stc_rtc_time_t stcTime;
    boolean_t bAlarmMatch = TRUE;

    stcWtcr1 = FM0P_RTC->WTCR1_f;

    /* Timer writing error Interrupt? */
    if (TRUE == stcWtcr1.INTERI)
    {
        /* Clear INTERI Flag */
        stcWtcr1.INTERI = FALSE;

        if (NULL != stcRtcInternData.pfnTimeWrtErrCallback)
        {
            /* Callback */
            stcRtcInternData.pfnTimeWrtErrCallback();
        }
    }

    /* Alarm Completion? */
    if (TRUE == stcWtcr1.INTALI)
    {
        /* Clear INTALI Flag */
        stcWtcr1.INTALI = FALSE;
        
        Rtc_ReadDateTime(&stcTime);
        
        if(FM0P_RTC->WTCR1_f.MIEN)
        {
            if(FM0P_RTC->ALMIR != DecToBcd(stcTime.u8Minute)) bAlarmMatch = FALSE;
        }
        if(FM0P_RTC->WTCR1_f.HEN)
        {
            if(FM0P_RTC->ALHR != DecToBcd(stcTime.u8Hour)) bAlarmMatch = FALSE;
        }
        if(FM0P_RTC->WTCR1_f.DEN)
        {
            if(FM0P_RTC->ALDR != DecToBcd(stcTime.u8Day)) bAlarmMatch = FALSE;
        }
        if(FM0P_RTC->WTCR1_f.MOEN)
        {
            if(FM0P_RTC->ALMOR != DecToBcd(stcTime.u8Month)) bAlarmMatch = FALSE;
        }
        if(FM0P_RTC->WTCR1_f.YEN)
        {
            if(FM0P_RTC->ALYR != DecToBcd(stcTime.u16Year - 2000)) bAlarmMatch = FALSE;
        }

        if(bAlarmMatch == TRUE)
        {
            if (NULL != stcRtcInternData.pfnAlarmCallback)
            {
                /* Callback */
                stcRtcInternData.pfnAlarmCallback();
            }
        }
    }

    /* Timer Interrupt? */
    if (TRUE == stcWtcr1.INTTMI)
    {
        /* Clear INTTMI Flag */
        stcWtcr1.INTTMI = FALSE;

        if (NULL != stcRtcInternData.pfnTimerCallback)
        {
            /* Callback */
            stcRtcInternData.pfnTimerCallback();
        }
    }

    /* 0.5-Second Interrupt? */
    if (TRUE == stcWtcr1.INTSSI)
    {
        /* Clear INTSSI Flag */
        stcWtcr1.INTSSI = FALSE;

        if (NULL != stcRtcInternData.pfnHalfSecondCallback)
        {
            /* Callback */
            stcRtcInternData.pfnHalfSecondCallback();
        }
    }

    /* One Second Interrupt? */
    if (TRUE == stcWtcr1.INTSI)
    {
        /* Clear INTSI Flag */
        stcWtcr1.INTSI = FALSE;

        if (NULL != stcRtcInternData.pfnOneSecondCallback)
        {
            /* Callback */
            stcRtcInternData.pfnOneSecondCallback();
        }
    }

    /* One Minute Interrupt? */
    if (TRUE == stcWtcr1.INTMI)
    {
        /* Clear INTMI Flag */
        stcWtcr1.INTMI = FALSE;

        if (NULL != stcRtcInternData.pfnOneMinuteCallback)
        {
            /* Callback */
            stcRtcInternData.pfnOneMinuteCallback();
        }
    }

    /* One Hour Interrupt? */
    if (TRUE == stcWtcr1.INTHI)
    {
        /* Clear INTHI Flag */
        stcWtcr1.INTHI = FALSE;

        if (NULL != stcRtcInternData.pfnOneHourCallback)
        {
            /* Callback */
            stcRtcInternData.pfnOneHourCallback();
        }
    }

    FM0P_RTC->WTCR1_f = stcWtcr1;
} /* RtcIrqHandler */

/**
 ******************************************************************************
 ** \brief Enable RTC (and Timer) Interrupts
 **
 ** \param [in]  pstcIntSel       Pointer to RTC interrupt selection structure
 ** \param [in]  pstcIntCb        Pointer to RTC interrupt callback functions structure
 **
 ** \retval Ok                    Interrupts are enabled
 ** \retval ErrorInvalidParameter If one of following conditions are met:
 **                               - pstcIntSel == NULL
 **                               - pstcIntCb  == NULL
 **
 ******************************************************************************/
en_result_t Rtc_EnableInt(stc_rtc_int_sel_t* pstcIntSel, stc_rtc_int_cb_t* pstcIntCb)
{
    stc_rtc_wtcr1_field_t stcWtcr1 ;

    stcWtcr1 = FM0P_RTC->WTCR1_f;
    /* Check for valid pointer */
    if ((NULL == pstcIntSel) || (NULL == pstcIntCb))
    {
        return ErrorInvalidParameter;
    }
    /* Enable timer rewrite error interrupt */
    if (TRUE == pstcIntSel->TimeRewriteErrorInt)
    {
        stcWtcr1.INTERIE = TRUE;
        stcRtcInternData.pfnTimeWrtErrCallback = pstcIntCb->pfnTimeWrtErrCallback;
    }

    /* Enable alarm coincidence interrupt */
    if (TRUE == pstcIntSel->AlarmInt)
    {
        stcWtcr1.INTALIE = TRUE;
        stcRtcInternData.pfnAlarmCallback = pstcIntCb->pfnAlarmCallback;
    }

    /* Enable timer underflow detection interrupt */
    if (TRUE == pstcIntSel->TimerInt)
    {
        stcWtcr1.INTTMIE = TRUE;
        stcRtcInternData.pfnTimerCallback = pstcIntCb->pfnTimerCallback;
    }

    /* Enable every hour interrupt */
    if (TRUE == pstcIntSel->OneHourInt)
    {
        stcWtcr1.INTHIE = TRUE;
        stcRtcInternData.pfnOneHourCallback = pstcIntCb->pfnOneHourCallback;
    }

    /* Enable every minute interrupt */
    if (TRUE == pstcIntSel->OneMinuteInt)
    {
        stcWtcr1.INTMIE = TRUE;
        stcRtcInternData.pfnOneMinuteCallback = pstcIntCb->pfnOneMinuteCallback;
    }

    /* Enable every second interrupt */
    if (TRUE == pstcIntSel->OneSecondInt)
    {
        stcWtcr1.INTSIE = TRUE;
        stcRtcInternData.pfnOneSecondCallback = pstcIntCb->pfnOneSecondCallback;
    }

    /* Enable every 0.5 second interrupt */
    if (TRUE == pstcIntSel->HalfSecondInt)
    {
        stcWtcr1.INTSSIE = TRUE;
        stcRtcInternData.pfnHalfSecondCallback = pstcIntCb->pfnHalfSecondCallback;
    }

    FM0P_RTC->WTCR1_f = stcWtcr1;  /* Update WTCR13 */

    RtcInitIrq();

    return (Ok);
} /* Rtc_EnableDisableInterrupts */

/**
 ******************************************************************************
 ** \brief Disable RTC (and Timer) Interrupts
 **
 ** \param [in]  pstcIntSel       Pointer to RTC interrupt selection structure
 **
 ** \retval Ok                    Interrupts are disabled
 ** \retval ErrorInvalidParameter pstcIntSel == NULL
 **
 ******************************************************************************/
en_result_t Rtc_DisableInt(stc_rtc_int_sel_t* pstcIntSel)
{
    stc_rtc_wtcr1_field_t stcWtcr1 ; 
    stcWtcr1 = FM0P_RTC->WTCR1_f;
    
    /* Check for valid pointer */
    if (NULL == pstcIntSel)
    {
        return ErrorInvalidParameter;
    }

    /* Enable timer rewrite error interrupt */
    if (TRUE == pstcIntSel->TimeRewriteErrorInt)
    {
        stcWtcr1.INTERIE = FALSE;
    }

    /* Enable alarm coincidence interrupt */
    if (TRUE == pstcIntSel->AlarmInt)
    {
        stcWtcr1.INTALIE = FALSE;
    }

    /* Enable timer underflow detection interrupt */
    if (TRUE == pstcIntSel->TimerInt)
    {
        stcWtcr1.INTTMIE = FALSE;
    }

    /* Enable every hour interrupt */
    if (TRUE == pstcIntSel->OneHourInt)
    {
        stcWtcr1.INTHIE = FALSE;
    }

    /* Enable every minute interrupt */
    if (TRUE == pstcIntSel->OneMinuteInt)
    {
        stcWtcr1.INTMIE = FALSE;;
    }

    /* Enable every second interrupt */
    if (TRUE == pstcIntSel->OneSecondInt)
    {
        stcWtcr1.INTSIE = FALSE;
    }

    /* Enable every 0.5 second interrupt */
    if (TRUE == pstcIntSel->HalfSecondInt)
    {
        stcWtcr1.INTSSIE = FALSE;
    }

    FM0P_RTC->WTCR1_f = stcWtcr1;  /* Update WTCR13 */

    if((FM0P_RTC->WTCR1 & 0xFF000000u) != 0u)
    {
        return Ok;
    }
    
    if(FM0P_WC->WCCR_f.WCIE != 0)
    {
        return Ok;
    }
        
    /* Only when RTC and WC interrupt are all disabled, disable IRQ */    
    RtcDeInitIrq();    

    return Ok;
} /* Rtc_DisableInt */

#endif

/**
 ******************************************************************************
 ** \brief Initialize RTC
 **
 ** This function initializes the RTC module 
 **
 ** \param [in] pstcConfig        Pointer to RTC configuration structure
 **
 ** \retval Ok                    RTC initialization completed normally
 ** \retval ErrorInvalidParameter If one of following conditions are met:
 **                               - pstcConfig == NULL
 **                               - Invalid value of a pstcConfig's element
 **
 ******************************************************************************/
en_result_t Rtc_Init(stc_rtc_config_t*       pstcConfig)
{
    uint32_t u32Timeout = RTC_TIMEOUT;
    
    if(pstcConfig == NULL)
    {
        return ErrorInvalidParameter;
    }
    
    /* For initialization: Force all bits of WTCR1 and WTCR2 (inclusive ST bit) */
    /*   to '0'  */
    FM0P_RTC->WTCR1 = 0;
    FM0P_RTC->WTCR2 = 0;

    /* Select source clock for RTC */
    switch (pstcConfig->enClkSel)
    {
        case RtcUseSubClk:
            /* Select sub clock as source clock */
            FM0P_RTC->WTCLKS_f.WTCLKS = 0u;
            /* Wait until sub clock stable */
            while(1)
            {
                if(FM0P_RTC->WTCLKM_f.WTCLKM == 0x02u)
                {
                    break;
                }
                else
                {
                    u32Timeout--;
                    if(u32Timeout == 0)
                    {
                        return ErrorTimeout;
                    }
                }
            }
            
            FM0P_RTC->WTBR = (__CLKSO/4) - 1; /* set 0.5s as sub-second interval */
            break;
        case RtcuseMainClk:
            /* Select main clock as source clock */
            FM0P_RTC->WTCLKS_f.WTCLKS = 1u;
            /* Wait until main clock stable */
            while(1)
            {
                if(FM0P_RTC->WTCLKM_f.WTCLKM == 0x03u)
                {
                    break;
                }
                else
                {
                    u32Timeout--;
                    if(u32Timeout == 0)
                    {
                        return ErrorTimeout;
                    }
                }
            }
            FM0P_RTC->WTBR = (__CLKMO/4) - 1; /* set 0.5s as sub-second interval */
            break;  
        default:
            return ErrorInvalidParameter;
    }
       
    if(pstcConfig->bEnSuboutDivider == TRUE)
    {
        /* Disable divider */
        FM0P_RTC->WTDIVEN_f.WTDIVEN = 0;
        
        /* Set Divider ratio */
        switch (pstcConfig->enDividerRatio)
        {
            case RtcDivRatio1:
            case RtcDivRatio2:
            case RtcDivRatio4:
            case RtcDivRatio8:
            case RtcDivRatio16:
            case RtcDivRatio32:
            case RtcDivRatio64:
            case RtcDivRatio128:
            case RtcDivRatio256:
            case RtcDivRatio512:
            case RtcDivRatio1024:
            case RtcDivRatio2048:
            case RtcDivRatio4096:
            case RtcDivRatio8192:
            case RtcDivRatio16384:
            case RtcDivRatio32768:
                FM0P_RTC->WTDIV = pstcConfig->enDividerRatio;
                break;
            default:
                return  ErrorInvalidParameter;
        }
        
        /*  Enable divider */
        u32Timeout = RTC_TIMEOUT;
        FM0P_RTC->WTDIVEN_f.WTDIVEN = 1;
        while(TRUE != FM0P_RTC->WTDIVEN_f.WTDIVRDY)
        {
            u32Timeout--;
            if(u32Timeout == 0)
            {
                return ErrorInvalidParameter;
            }
        }
    }
    
    /* Initial frequency correction module */
    if(pstcConfig->bInitFreqCorr == TRUE)
    {
        if(pstcConfig->stcFreqCorrConfig.u16FreqCorrValue > RTC_MAX_FREQ_CORR_VALUE)
        {
            return ErrorInvalidParameter;
        }
        FM0P_RTC->WTCAL = pstcConfig->stcFreqCorrConfig.u16FreqCorrValue;
        
        if(pstcConfig->stcFreqCorrConfig.u16FreqCorrCycle > RTC_MAX_FREQ_CORR_CYCLE_SET_VALUE)
        {
            return ErrorInvalidParameter;
        }
        FM0P_RTC->WTCALPRD = pstcConfig->stcFreqCorrConfig.u16FreqCorrCycle;
    }
    
    /* Configure RTCCO output */
    if (pstcConfig->enRtccoSel == RtccoOutput2Hz) 
    {
        FM0P_RTC->WTCOSEL_f.WTCOSEL = 0; 
    }
    else
    {
        FM0P_RTC->WTCOSEL_f.WTCOSEL = 1; 
    }
      
    /* Initialize time and date */
    if(pstcConfig->bInitTimeDate == TRUE)
    {
        FM0P_RTC->WTSR  = DecToBcd(pstcConfig->stcTimeDate.u8Second);
        FM0P_RTC->WTMIR = DecToBcd(pstcConfig->stcTimeDate.u8Minute);
        FM0P_RTC->WTHR  = DecToBcd(pstcConfig->stcTimeDate.u8Hour);
        FM0P_RTC->WTDR  = DecToBcd(pstcConfig->stcTimeDate.u8Day);
        FM0P_RTC->WTMOR = DecToBcd(pstcConfig->stcTimeDate.u8Month);
        FM0P_RTC->WTYR  = DecToBcd(pstcConfig->stcTimeDate.u16Year - 2000);
        FM0P_RTC->WTDW  = DecToBcd(pstcConfig->stcTimeDate.u8DayOfWeek);
    }
    
    /* Initialize Alarm */
    if(pstcConfig->bInitAlarm == TRUE)
    {
        FM0P_RTC->ALMIR = DecToBcd(pstcConfig->stcAlarm.u8Minute);
        FM0P_RTC->ALHR  = DecToBcd(pstcConfig->stcAlarm.u8Hour);
        FM0P_RTC->ALDR  = DecToBcd(pstcConfig->stcAlarm.u8Day);
        FM0P_RTC->ALMOR = DecToBcd(pstcConfig->stcAlarm.u8Month);
        FM0P_RTC->ALYR  = DecToBcd(pstcConfig->stcAlarm.u16Year - 2000);
    }
    
    if(pstcConfig->bInitTimer == TRUE)
    {
        switch (pstcConfig->stcTimer.enMode)
        {
            case RtcTimerOneshot:
                FM0P_RTC->WTCR2_f.TMEN = 0u;
                break;
            case RtcTimerPeriod:
                FM0P_RTC->WTCR2_f.TMEN = 1u;
                break;
            default:
                return ErrorInvalidParameter;
        }
        
        if(pstcConfig->stcTimer.u32TimerCycle > RTC_MAX_TIMER_SET_VALUE)
        {
            return ErrorInvalidParameter;
        }
        
        FM0P_RTC->WTTR = (pstcConfig->stcTimer.u32TimerCycle*2) - 1;
        
    }

    return (Ok);
} /* Rtc_Init */

/**
 ******************************************************************************
 ** \brief De-Initialize RTC
 **
 ** This function stops and resets the RTC module and its interrupts.
 **
 ** \retval  Ok                     RTC de-initialization normally
 **
 ******************************************************************************/
en_result_t Rtc_DeInit(void)
{
    /* Set all registers of RTC instance to '0' */
    FM0P_RTC->WTCAL    = 0;
    FM0P_RTC->WTCALEN  = 0;
    FM0P_RTC->WTDIV    = 0;
    FM0P_RTC->WTDIVEN  = 0;
    FM0P_RTC->WTCALPRD = 0;
    FM0P_RTC->WTCOSEL  = 0;
    FM0P_RTC->WTSR     = 0;
    FM0P_RTC->WTMIR    = 0;
    FM0P_RTC->WTHR     = 0;
    FM0P_RTC->WTDR     = 0;
    FM0P_RTC->WTDW     = 0;
    FM0P_RTC->WTMOR    = 0;
    FM0P_RTC->WTYR     = 0;
    FM0P_RTC->ALMIR    = 0;
    FM0P_RTC->ALHR     = 0;
    FM0P_RTC->ALDR     = 0;
    FM0P_RTC->ALMOR    = 0;
    FM0P_RTC->ALYR     = 0;
        
    return (Ok);
} /* Rtc_DeInit */


/**
 ******************************************************************************
 ** \brief Enable functions of RTC
 **
 ** These functions includes RTC counting, Timer, Alarm compariron and frequency
 ** correction module.
 **
 ** \param enFunc  Function types
 ** \arg   RtcCount          RTC counting
 ** \arg   RtcTimer          RTC timer
 ** \arg   RtcFreqCorr       RTC frequency correction module
 ** \arg   RtcAlarmYearEn    Year comparison enable of RTC alarm
 ** \arg   RtcAlarmMonthEn   Month comparison enable of RTC alarm
 ** \arg   RtcAlarmDayEn     Day comparison enable of RTC alarm
 ** \arg   RtcAlarmHourEn    Hour comparison enable of RTC alarm 
 ** \arg   RtcAlarmMinEn     Minute comparison enable of RTC alarm
 **
 ** \retval Ok                    RTC functions enabled normally
 ** \retval ErrorInvalidParameter If one of following conditions are met:
 **                               - Invalid value of enFunc 
 **
 ******************************************************************************/
en_result_t Rtc_EnableFunc(en_rtc_func_t enFunc)
{
    switch (enFunc)
    {
        case RtcCount:
            FM0P_RTC->WTCR1_f.ST = 1u;
            break;
        case RtcTimer:
            FM0P_RTC->WTCR2_f.TMST = 1u;
            break;
        case RtcFreqCorr:
            FM0P_RTC->WTCALEN_f.WTCALEN = 1u;
            break;
        case RtcAlarmYearEn:
            FM0P_RTC->WTCR1_f.YEN = 1u;
            break;
        case RtcAlarmMonthEn:
            FM0P_RTC->WTCR1_f.MOEN = 1u;
            break;
        case RtcAlarmDayEn:
            FM0P_RTC->WTCR1_f.DEN = 1u;
            break;
        case RtcAlarmHourEn:
            FM0P_RTC->WTCR1_f.HEN = 1u;
            break;
        case RtcAlarmMinEn:
            FM0P_RTC->WTCR1_f.MIEN = 1u;
            break;
        default:
            return ErrorInvalidParameter;
    }
    
    return Ok;
}
       
/**
 ******************************************************************************
 ** \brief Disable functions of RTC
 **
 ** These functions includes RTC counting, Timer, Alarm compariron and frequency
 ** correction module.
 **
 ** \param enFunc  Function types
 ** \arg   RtcCount          RTC counting
 ** \arg   RtcTimer          RTC timer
 ** \arg   RtcFreqCorr       RTC frequency correction module
 ** \arg   RtcAlarmYearEn    Year comparison enable of RTC alarm
 ** \arg   RtcAlarmMonthEn   Month comparison enable of RTC alarm
 ** \arg   RtcAlarmDayEn     Day comparison enable of RTC alarm
 ** \arg   RtcAlarmHourEn    Hour comparison enable of RTC alarm 
 ** \arg   RtcAlarmMinEn     Minute comparison enable of RTC alarm
 **
 ** \retval Ok                    RTC functions disabled normally
 ** \retval ErrorInvalidParameter If one of following conditions are met:
 **                               - Invalid value of enFunc 
 **
 ******************************************************************************/
en_result_t Rtc_DisableFunc(en_rtc_func_t enFunc)
{
    switch (enFunc)
    {
        case RtcCount:
            FM0P_RTC->WTCR1_f.ST = 0u;
            break;
        case RtcTimer:
            FM0P_RTC->WTCR2_f.TMST = 0u;
            break;
        case RtcFreqCorr:
            FM0P_RTC->WTCALEN_f.WTCALEN = 0u;
            break;
        case RtcAlarmYearEn:
            FM0P_RTC->WTCR1_f.YEN = 0u;
            break;
        case RtcAlarmMonthEn:
            FM0P_RTC->WTCR1_f.MOEN = 0u;
            break;
        case RtcAlarmDayEn:
            FM0P_RTC->WTCR1_f.DEN = 0u;
            break;
        case RtcAlarmHourEn:
            FM0P_RTC->WTCR1_f.HEN = 0u;
            break;
        case RtcAlarmMinEn:
            FM0P_RTC->WTCR1_f.MIEN = 0u;
            break;
        default:
            return ErrorInvalidParameter;
    }
    
    return Ok;
}

/**
 ******************************************************************************
 ** \brief Get status of RTC according to status type
 **
 ** \param [in] enStatus           RTC status type
 ** \arg   RtcRewriteError         RTC rewrite error
 ** \arg   RtcAlarmElementMatch    RTC alarm match
 ** \arg   RtcTimerUnderFlow       RTC Timer underflow
 ** \arg   RtcOneHourFlag          1-hour count-up flag 
 ** \arg   RtcOneMiniteFlag        1-Minute count-up flag
 ** \arg   RtcOneSecondFlag        1-second count-up flag
 ** \arg   RtcHalfSecondFlag       0.5-second count-up flag
 ** \arg   RtcRunStatus            RTC counter run status
 ** \arg   RtcTimerStatus          RTC timer status
 ** 
 ** \retval FALSE                 If one of following conditions are met:
 **                               - No RTC rewrite error [enStatus = RtcRewriteError]
 **                               - No element of alarm matchs [enStatus = RtcAlarmElementMatch]     
 **                               - RTC timer underflow doesn't occurs [enStatus = RtcTimerUnderFlow]
 **                               - 1-hour count-up flag is not set [enStatus = RtcOneHourFlag]
 **                               - 1-minute count-up flag is not set [enStatus = RtcOneMiniteFlag]
 **                               - 1-second count-up flag is not set [enStatus = RtcOneSecondFlag]
 **                               - 0.5-second count-up flag is not set [enStatus = RtcHalfSecondFlag]
 **                               - RTC is running [enStatus = RtcRunStatus]
 **                               - RTC timer is running [enStatus = RtcTimerStatus]
 ** \retval TRUE                 If one of following conditions are met:
 **                               - RTC rewrite error occurs [enStatus = RtcRewriteError]
 **                               - One of an element of alarm matchs [enStatus = RtcAlarmElementMatch]     
 **                               - RTC timer underflow doesn't occurs [enStatus = RtcTimerUnderFlow]
 **                               - 1-hour count-up flag is set [enStatus = RtcOneHourFlag]
 **                               - 1-minute count-up flag is set [enStatus = RtcOneMiniteFlag]
 **                               - 1-second count-up flag is set [enStatus = RtcOneSecondFlag]
 **                               - 0.5-second count-up flag is set [enStatus = RtcHalfSecondFlag]
 **                               - RTC counter stops [enStatus = RtcRunStatus]
 **                               - RTC timer stops [enStatus = RtcTimerStatus]
 ** 
 ******************************************************************************/
boolean_t Rtc_GetStatus(en_rtc_status_t enStatus)
{
    boolean_t bRet = FALSE;
      
    switch(enStatus)
    {
        case RtcRewriteError:
            (FM0P_RTC->WTCR1_f.INTERI == 1) ? (bRet = TRUE) : (bRet = FALSE);
            break;
        case RtcAlarmElementMatch:
            (FM0P_RTC->WTCR1_f.INTALI == 1) ? (bRet = TRUE) : (bRet = FALSE);
            break;
        case RtcTimerUnderFlow:
            (FM0P_RTC->WTCR1_f.INTTMI == 1) ? (bRet = TRUE) : (bRet = FALSE);
            break;
        case RtcOneHourFlag:
            (FM0P_RTC->WTCR1_f.INTHI == 1) ? (bRet = TRUE) : (bRet = FALSE);
            break;
        case RtcOneMiniteFlag:
            (FM0P_RTC->WTCR1_f.INTMI == 1) ? (bRet = TRUE) : (bRet = FALSE);
            break;
        case RtcOneSecondFlag:
            (FM0P_RTC->WTCR1_f.INTSI == 1) ? (bRet = TRUE) : (bRet = FALSE);
            break;
        case RtcHalfSecondFlag:
            (FM0P_RTC->WTCR1_f.INTSSI == 1) ? (bRet = TRUE) : (bRet = FALSE);
            break;
        case RtcRunStatus:
            (FM0P_RTC->WTCR1_f.RUN == 1) ? (bRet = TRUE) : (bRet = FALSE);
            break;
        case RtcTimerStatus:
            (FM0P_RTC->WTCR2_f.TMRUN == 1) ? (bRet = TRUE) : (bRet = FALSE);
            break;
        default:
            break;  
    }
    
    return bRet;
}

/**
 ******************************************************************************
 ** \brief Clear status of RTC according to status type
 **
 ** \param [in] enStatus           RTC status type
 ** \arg   RtcRewriteError         RTC rewrite error
 ** \arg   RtcAlarmElementMatch    RTC alarm match
 ** \arg   RtcTimerUnderFlow       RTC Timer underflow
 ** \arg   RtcOneHourFlag          1-hour count-up flag 
 ** \arg   RtcOneMiniteFlag        1-Minute count-up flag
 ** \arg   RtcOneSecondFlag        1-second count-up flag
 ** \arg   RtcHalfSecondFlag       0.5-second count-up flag
 ** \arg   RtcRunStatus            RTC counter run status
 ** \arg   RtcTimerStatus          RTC timer status
 ** 
 ** \retval Ok                    RTC status is cleared normally
 ** \retval ErrorInvalidParameter If one of following conditions are met:
 **                               - Invalid value of enStatus 
 ** \note  The following status can only be cleared by hardware behavior:
 **        - RtcRunStatus
 **        - RtcTimerStatus
 ** 
 ******************************************************************************/
en_result_t Rtc_ClrStatus(en_rtc_status_t enStatus)
{      
    switch(enStatus)
    {
        case RtcRewriteError:
            FM0P_RTC->WTCR1_f.INTERI = 0;
            break;
        case RtcAlarmElementMatch:
            FM0P_RTC->WTCR1_f.INTALI = 0;
            break;
        case RtcTimerUnderFlow:
            FM0P_RTC->WTCR1_f.INTTMI = 0;
            break;
        case RtcOneHourFlag:
            FM0P_RTC->WTCR1_f.INTHI = 0;
            break;
        case RtcOneMiniteFlag:
            FM0P_RTC->WTCR1_f.INTMI = 0;
            break;
        case RtcOneSecondFlag:
            FM0P_RTC->WTCR1_f.INTSI = 0;
            break;
        case RtcHalfSecondFlag:
            FM0P_RTC->WTCR1_f.INTSSI = 0;
            break;
        case RtcRunStatus:
        case RtcTimerStatus:  
            break;
        default:
            return ErrorInvalidParameter;  
    }
    
    return Ok;
}
        
/**
 ******************************************************************************
 ** \brief Reset RTC
 ******************************************************************************/
void Rtc_Reset(void)
{
    FM0P_RTC->WTCR1_f.SRST = 1;
}
        
/**
 ******************************************************************************
 ** \brief Set time and date after RTC is running
 **
 ** \param [in] pstcTimeDate      Time and date structure
 ** \param [in] bContinue         RTC counting continues or not
 ** \arg  FALSE     RTC 1-second counting reset during setting procedure
 ** \arg  TRUE      RTC 1-second continues during setting procedure
 ** 
 ** \retval Ok                    RTC time and date is set normally
 ** \retval ErrorInvalidParameter If one of following conditions are met:
 **                               - pstcTimeDate == NULL 
 ** \retval ErrorInvalidMode      RTC is running
 ** \retval ErrorTimeout          RTC rewrite timeout            
 ** 
 ******************************************************************************/
en_result_t Rtc_SetDateTime(stc_rtc_time_t* pstcTimeDate, boolean_t bContinue)
{
    uint32_t u32TimeOut;
  
     /* Don't set time and date with this function before RTC starts */
    if(FM0P_RTC->WTCR1_f.RUN == 1)
    {
        return ErrorInvalidMode;
    }
    
    if(pstcTimeDate == NULL)
    {
        return ErrorInvalidParameter;  
    }

    if(FM0P_RTC->WTCR1_f.BUSY == 1)
    {
        /*  Wait until rewrite idle */
        u32TimeOut = RTC_TIMEOUT;
        while(1)
        {
            if(bContinue == TRUE)
            {
                if((FM0P_RTC->WTCR1_f.BUSY == 0) && (FM0P_RTC->WTCR1_f.SCST == 0))
                {
                    break;
                }
            }
            else
            {
                if((FM0P_RTC->WTCR1_f.BUSY == 0) && (FM0P_RTC->WTCR1_f.SCRST == 0))
                {
                    break;
                }
            }
            u32TimeOut--;
            if(u32TimeOut == 0)
            {
                return ErrorTimeout;
            }
        }       
    }
             
    RtcDisableNvic();
                    
    if(bContinue == TRUE)
    {
        /* Stop 1 second output */
        FM0P_RTC->WTCR1_f.SCST = 1;
    }
    else
    {
        /* Reset 1 second counter */
        FM0P_RTC->WTCR1_f.SCRST = 1;
    }
    
    /* Set time and date */
    FM0P_RTC->WTSR  = DecToBcd(pstcTimeDate->u8Second);
    FM0P_RTC->WTMIR = DecToBcd(pstcTimeDate->u8Minute);
    FM0P_RTC->WTHR  = DecToBcd(pstcTimeDate->u8Hour);
    FM0P_RTC->WTDR  = DecToBcd(pstcTimeDate->u8Day);
    FM0P_RTC->WTMOR = DecToBcd(pstcTimeDate->u8Month);
    FM0P_RTC->WTYR  = DecToBcd(pstcTimeDate->u16Year - 2000);
    FM0P_RTC->WTDW  = DecToBcd(pstcTimeDate->u8DayOfWeek);
    
    if(bContinue == TRUE)
    {
        /* Refresh the time and date */
        FM0P_RTC->WTCR1_f.SCST = 0;
    }
    else
    {
        /* Refresh the time and date  */
        FM0P_RTC->WTCR1_f.SCRST = 0;
    }
    
    RtcRestoreNvic();    
    
    /* Wait until the transfer from time and date registers to counters end */
    u32TimeOut = RTC_TIMEOUT;
    while(1)
    {
        if(bContinue == TRUE)
        {
            if((FM0P_RTC->WTCR1_f.BUSY == 0) && (FM0P_RTC->WTCR1_f.SCST == 0))
            {
                break;
            }
        }
        else
        {
            if((FM0P_RTC->WTCR1_f.BUSY == 0) && (FM0P_RTC->WTCR1_f.SCRST == 0))
            {
                break;
            }
        }
        u32TimeOut--;
        if(u32TimeOut == 0)
        {
            return ErrorTimeout;
        }
    }    
                 
    return Ok;            
}
                
/**
 ******************************************************************************
 ** \brief Read RTC time and date 
 **
 ** \param [in] pstcTimeDate      Time and date structure
 ** 
 ** \retval Ok                    RTC time and date is read normally
 ** \retval ErrorInvalidParameter If one of following conditions are met:
 **                               - pstcTimeDate == NULL 
 ** \retval ErrorTimeout          Time and date read timeout
 ** 
 ******************************************************************************/
en_result_t Rtc_ReadDateTime(stc_rtc_time_t* pstcTimeDate)
{
    uint32_t u32TimeOut;
    uint8_t  u8DayOfWeek, u8BcdSec, u8BcdMin, u8BcdHour, u8Day, u8Month, u16Year = 0;
  
    if(pstcTimeDate == NULL)
    {
        return ErrorInvalidParameter;
    }
    
    RtcDisableNvic();
    
    if(FM0P_RTC->WTCR2_f.CREAD == 1) /* read is in process? */
    {
        ;  /* Already been read */
    }
    else
    {
        /* Start read */
        FM0P_RTC->WTCR2_f.CREAD = 1;

    }
    
    /* Wait until read finish */
    u32TimeOut = RTC_TIMEOUT;
    while(1)
    {
        if(FM0P_RTC->WTCR2_f.CREAD == 0)
        {
            break;
        }
        u32TimeOut --;
        if(u32TimeOut == 0)
        {
            RtcRestoreNvic(); 
            return ErrorTimeout;
        }
        
    }
    
    /* Read time and date */    
    u8BcdSec  = FM0P_RTC->WTSR;
    u8BcdMin  = FM0P_RTC->WTMIR;
    u8BcdHour = FM0P_RTC->WTHR;
    u8Day     = FM0P_RTC->WTDR;
    u8Month   = FM0P_RTC->WTMOR;
    u16Year    = FM0P_RTC->WTYR;
    u8DayOfWeek = FM0P_RTC->WTDW;
    
    pstcTimeDate->u8Second = BcdToDec(u8BcdSec);
    pstcTimeDate->u8Minute = BcdToDec(u8BcdMin);
    pstcTimeDate->u8Hour   = BcdToDec(u8BcdHour);
    pstcTimeDate->u8Day    = BcdToDec(u8Day);
    pstcTimeDate->u8Month  = BcdToDec(u8Month);
    pstcTimeDate->u16Year  = BcdToDec(u16Year) + 2000;
    pstcTimeDate->u8DayOfWeek = BcdToDec(u8DayOfWeek);
    
    RtcRestoreNvic(); 
    
    return Ok;
}

/**
 ******************************************************************************
 ** \brief Set time and date information of Alarm
 **
 ** \param [in] pstcAlarm         Time and date structure 
 ** \retval Ok                    RTC Alarm time and date is set normally
 ** \retval ErrorInvalidParameter If one of following conditions are met:
 **                               - pstcAlarm == NULL 
 ** 
 ******************************************************************************/
en_result_t Rtc_SetAlarmDateTime(stc_rtc_alarm_t* pstcAlarm)
{
    if(pstcAlarm == NULL)
    {
        return ErrorInvalidParameter;
    }
  
    FM0P_RTC->ALMIR = DecToBcd(pstcAlarm->u8Minute);
    FM0P_RTC->ALHR  = DecToBcd(pstcAlarm->u8Hour);
    FM0P_RTC->ALDR  = DecToBcd(pstcAlarm->u8Day);
    FM0P_RTC->ALMOR = DecToBcd(pstcAlarm->u8Month);
    FM0P_RTC->ALYR  = DecToBcd(pstcAlarm->u16Year - 2000);
    
    return Ok;
}
                
/**
 ******************************************************************************
 ** \brief Get time and date information of Alarm
 **
 ** \param [in] pstcAlarm         Time and date structure 
 ** \retval Ok                    RTC Alarm time and date is read normally
 ** \retval ErrorInvalidParameter If one of following conditions are met:
 **                               - pstcAlarm == NULL 
 ** 
 ******************************************************************************/
en_result_t Rtc_GetAlarmDateTime(stc_rtc_alarm_t* pstcAlarm)
{
    uint8_t  u8BcdMin, u8BcdHour, u8Day, u8Month, u8Year;
    if(pstcAlarm == NULL)
    {
        return ErrorInvalidParameter;
    }
    
    u8BcdMin  = FM0P_RTC->ALMIR;
    u8BcdHour = FM0P_RTC->ALHR;
    u8Day     = FM0P_RTC->ALDR;
    u8Month   = FM0P_RTC->ALMOR;
    u8Year    = FM0P_RTC->ALYR;
    
    pstcAlarm->u8Minute = BcdToDec(u8BcdMin);
    pstcAlarm->u8Hour   = BcdToDec(u8BcdHour);
    pstcAlarm->u8Day    = BcdToDec(u8Day);
    pstcAlarm->u8Month  = BcdToDec(u8Month);
    pstcAlarm->u16Year   = BcdToDec(u8Year) + 2000;
    
    return Ok;
}
                
/**
 ******************************************************************************
 ** \brief Set Day of the Week
 **
 ** This function calculates the day of the week from YY-MM-DD in the Time
 ** structure. It uses mktime of time.h library.
 **
 ** \param  [in,out] pstcRtcTime     RTC Time structure
 **
 ** \retval Ok                       Internal data has been setup
 ** \retval ErrorInvalidParameter    pstcRtcTime == NULL or mktime failed
 **
 ******************************************************************************/
en_result_t Rtc_SetDayOfWeek(stc_rtc_time_t* pstcRtcTime)
{
    en_result_t enResult;
    struct tm stcTime;

    enResult = ErrorInvalidParameter;

    /* Check for NULL pointer */
    if (NULL != pstcRtcTime)
    {
        enResult = Ok;
        /* Convert RTC time structure to time.h tm structure */
        RtcToTm(pstcRtcTime, &stcTime);

        /* Calculated raw time (UNIX time) is error */
        if ((time_t)RTC_ERR == mktime(&stcTime))
        {
            enResult = ErrorInvalidParameter;
        }

        if (Ok == enResult)
        {
            /* Set calculated the day of week */
            pstcRtcTime->u8DayOfWeek = (uint8_t)(stcTime.tm_wday);
        }
    }

    return (enResult);
} /* Rtc_SetDayOfWeek */
                
/**
 ******************************************************************************
 ** \brief Set cycle of RTC timer
 **
 ** \param [in] u32TimerCycle     Timer cycle
 ** \retval Ok                    RTC timer cycle is set normally
 ** \retval ErrorInvalidParameter If one of following conditions are met:
 **                               - u32TimerCycle > RTC_MAX_TIMER_SET_VALUE
 ** 
 ** \note The unit of parameter u32TimerCycle is second.
 ** 
 ******************************************************************************/
en_result_t Rtc_SetTimerCycle(uint32_t u32TimerCycle)
{
    if(u32TimerCycle > RTC_MAX_TIMER_SET_VALUE)
    {
        return ErrorInvalidParameter;
    }
  
    FM0P_RTC->WTTR = (u32TimerCycle*2) - 1;
    
    return Ok;
}
       
/**
 ******************************************************************************
 ** \brief Set calibration value of RTC frequency correction module
 **
 ** \param [in] u16Value          Calibration value
 ** \retval Ok                    RTC timer cycle is set normally
 ** \retval ErrorInvalidParameter If one of following conditions are met:
 **                               - u32TimerCycle > RTC_MAX_FREQ_CORR_VALUE
 ** 
 ******************************************************************************/
en_result_t Rtc_SetFreqCorrValue(uint16_t u16Value)
{
    if(u16Value > RTC_MAX_FREQ_CORR_VALUE)
    {
        return ErrorInvalidParameter;
    }
  
    FM0P_RTC->WTCAL = u16Value;
    
    return Ok;
}

//@} // RtcGroup

#endif /* #if (defined(PDL_PERIPHERAL_RTC_ACTIVE)) */
/******************************************************************************/
/* EOF (not truncated)                                                        */
/******************************************************************************/
